## Controller Python Script "correct_and_evaluate_deposit_file_script"
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=
##title=Cours edit
##

objet = context
form = context.REQUEST.form

correction = False
note = False
properties = {}
if "correction" in form:
    properties["Correction"] = form["correction"].strip()
    if form["correction"].strip() and form["correction"] != context.getCorrectionDepot():
        correction=True
if "note" in form:
    properties["Note"] = form["note"].strip()
    if form["note"].strip() and form["note"] != context.getNote():
        note=True
if properties:
    context.setProperties(properties)

if "file_file" in form and form["file_file"]:
    boite = context.aq_parent
    corrections = getattr(boite, "corrections", None)
    if not corrections:
        boite.invokeFactory(type_name='JalonFolder', id="corrections")
        corrections = getattr(boite, "corrections")
    try:
        idobj = corrections.invokeFactory(type_name='JalonFile', id="Correction_%s" % context.getId())
    except:
        idobj = "Correction_%s" % context.getId()
    obj = getattr(corrections, idobj)
    param = {"Title":       "%s (Correction)" % context.Title(),
             "Description": "",
             "File":        form["file_file"]}
    obj.setProperties(param)
    context.reindexObject()
    correction=True

if correction or note:
    context.notifierCorrection(correction, note)

return context.REQUEST.RESPONSE.redirect(context.aq_parent.absolute_url())



# -*- coding: utf-8 -*-
"""Scripts de vue pour "Mes cours"."""
from zope.component import getMultiAdapter
from Products.Five.browser import BrowserView
from Products.CMFCore.utils import getToolByName

from jalon.content import contentMessageFactory as _
from jalon.content.content import jalon_utils

# from logging import getLogger
# LOG = getLogger('[MyCoursesView]')


class MyCoursesView(BrowserView):
    """Classe de vue pour la page "Mes cours"."""

    def __init__(self, context, request):
        # LOG.info("----- Init -----")
        BrowserView.__init__(self, context, request)
        self.context = context
        self.request = request

    def isAnonymous(self):
        # LOG.info("----- isAnonymous -----")
        portal_state = getMultiAdapter((self.context, self.request),
                                       name=u'plone_portal_state')
        return portal_state.anonymous()

    def getBreadcrumbs(self):
        # LOG.info("----- getBreadcrumbs -----")
        return [{"title": _(u"Mes cours"),
                 "icon":  "fa fa-university",
                 "link":  self.context.absolute_url()}]

    def getUserFolder(self, user_id):
        # LOG.info("----- getUserFolder -----")
        portal_state = getMultiAdapter((self.context, self.request), name=u'plone_portal_state')
        portal = portal_state.portal()

        return getattr(portal.cours, user_id)

    def getMyCoursesView(self, user, tab=None):
        # LOG.info("----- getMyCoursesView -----")

        folder = jalon_utils.getCourseUserFolder(self.context, user.getId())
        context_link = self.context.absolute_url()

        is_manager = user.has_role("Manager")
        is_student = user.has_role("Etudiant") or user.has_role("EtudiantJalon")

        macro_messages = "Etudiant"
        if is_manager:
            macro_messages = "Manager"
        if user.has_role("Personnel"):
            macro_messages = "Personnel"

        tabs_list = []
        actions_list = []
        if is_student:
            is_tab_password = True if tab == "2" else False
            my_courses_macro = "my_courses_list_student_macro"
            tab = tab if tab else "1"
            courses_dict = self.getStudentCoursesList(user, tab, folder, is_tab_password)
        else:
            my_courses_macro = "my_courses_list_teacher_macro"
            actions_list = [{"css_class":   "button create expand",
                             "action_link": "%s/create_course_form" % context_link,
                             "action_icon": "fa fa-plus-circle",
                             "action_name": "Créer un cours"},
                            {"css_class":   "button",
                             "action_link": "%s/choose_author_page" % context_link,
                             "action_icon": "fa fa-code-fork",
                             "action_name": "Dupliquer un cours"}]
            if not is_manager:
                del actions_list[-1]

            if "tab" in self.request:
                tab = self.request["tab"]
            if not tab:
                tab = "1" if folder.getComplement() == "True" else "2"
            tabs_list = [{"css_class": "button small selected" if tab == "1" else "button small secondary",
                          "tab_link":  "%s?tab=1" % context_link,
                          "tab_icon":  "fa fa-star fa-fw",
                          "tab_name":  "Favoris"},
                         {"css_class": "button small selected" if tab == "2" else "button small secondary",
                          "tab_link":  "%s?tab=2" % context_link,
                          "tab_icon":  "fa fa-user fa-fw",
                          "tab_name":  "Auteur"},
                         {"css_class": "button small selected" if tab == "3" else "button small secondary",
                          "tab_link":  "%s?tab=3" % context_link,
                          "tab_icon":  "fa fa-users fa-fw",
                          "tab_name":  "Co-auteur"},
                         {"css_class": "button small selected" if tab == "4" else "button small secondary",
                          "tab_link":  "%s?tab=4" % context_link,
                          "tab_icon":  "fa fa-graduation-cap fa-fw",
                          "tab_name":  "Lecteur"},
                         {"css_class": "button small selected" if tab == "5" else "button small secondary",
                          "tab_link":  "%s?tab=5" % context_link,
                          "tab_icon":  "fa fa-folder fa-fw",
                          "tab_name":  "Archives"}]

            courses_dict = self.getTeacherCoursesList(user, tab, folder)

        return {"macro_messages":   macro_messages,
                "my_courses_macro": my_courses_macro,
                "tab":              tab,
                "is_manager":       is_manager,
                "is_student":       is_student,
                "actions_list":     actions_list,
                "tabs_list":        tabs_list,
                "courses_dict":     courses_dict}

    def getTeacherCoursesList(self, member, tab, folder):
        """Renvoi la liste des cours pour l'enseignant member."""
        # LOG.info("----- getTeacherCoursesList -----")
        courses_list = []
        courses_ids_list = []
        courses_list_filter = []
        member_id = member.getId()
        member_login_time = member.getProperty('login_time', None)
        portal_catalog = getToolByName(self, "portal_catalog")

        actions_list = [{"action_link": "/add_favorite_course_form?course_id=",
                         "action_icon": "fa fa-star fa-fw",
                         "action_name": "Ajouter aux favoris"},
                        {"action_link": "/access_statistics_course?course_id=",
                         "action_icon": "fa fa-bar-chart fa-fw",
                         "action_name": "Voir les statistiques"},
                        {"action_link": "/duplicate_course_form?tab=%s&amp;course_id=" % tab,
                         "action_icon": "fa fa-code-fork fa-fw",
                         "action_name": "Dupliquer"},
                        {"action_link": "/add_archive_course_form?tab=%s&amp;course_id=" % tab,
                         "action_icon": "fa fa-folder fa-fw",
                         "action_name": "Archiver ce cours"},
                        {"action_link": "/delete_course_form?tab=%s&amp;course_id=" % tab,
                         "action_icon": "fa fa-trash-o fa-fw",
                         "action_name": "Supprimer ce cours"}]
        """
                        {"action_link": "/purge_course_form?tab=%s&amp;course_id=" % tab,
                         "action_icon": "fa fa-filter fa-fw",
                         "action_name": "Purger les travaux étudiants"},
                        {"action_link": "/delete_wims_activity_form?tab=%s&amp;course_id=" % tab,
                         "action_icon": "fa fa-trash-o fa-fw",
                         "action_name": "Supprimer les activités WIMS"},
        """

        filtre = {"portal_type": "JalonCours"}
        if tab == "1":
            filtre["Subject"] = member_id
            courses_list = list(portal_catalog.searchResults(portal_type="JalonCours", getAuteurPrincipal=member_id, Subject=member_id))
            courses_list.extend(list(portal_catalog.searchResults(
                portal_type="JalonCours", getCoAuteurs=member_id, Subject=member_id)))
            courses_list.extend(list(portal_catalog.searchResults(
                portal_type="JalonCours", getCoLecteurs=member_id, Subject=member_id)))
            courses_list.extend(list(folder.getFolderContents(contentFilter=filtre)))
            actions_list[0] = {"action_link": "/remove_favorite_course_form?course_id=",
                               "action_icon": "fa fa-star-o fa-fw warning",
                               "action_name": "Retirer des favoris"}
            if len(courses_list):
                folder.setComplement("True")
            else:
                folder.setComplement("False")
        if tab == "2":
            courses_list = list(portal_catalog.searchResults(
                portal_type="JalonCours", getAuteurPrincipal=member_id))
            courses_list.extend(list(folder.getFolderContents(contentFilter=filtre)))
        if tab == "3":
            courses_list.extend(
                list(portal_catalog.searchResults(portal_type="JalonCours", getCoAuteurs=member_id)))
            del actions_list[2]
            del actions_list[-1]
        if tab == "4":
            courses_list.extend(
                list(portal_catalog.searchResults(portal_type="JalonCours", getCoLecteurs=member_id)))
            actions_list = [{"action_link": "/display_course_page?course_id=",
                             "action_icon": "fa fa-book fa-fw",
                             "action_name": "Accéder à ce cours"}]
        if tab == "5":
            filtre["getArchive"] = member_id
            courses_list = list(portal_catalog.searchResults(
                portal_type="JalonCours", getAuteurPrincipal=member_id, getArchive=member_id))
            courses_list.extend(list(portal_catalog.searchResults(
                portal_type="JalonCours", getCoAuteurs=member_id, getArchive=member_id)))
            courses_list.extend(list(portal_catalog.searchResults(
                portal_type="JalonCours", getCoLecteurs=member_id, getArchive=member_id)))
            courses_list.extend(list(folder.getFolderContents(contentFilter=filtre)))
            actions_list[-2] = {"action_link": "/remove_archive_course_form?course_id=",
                                "action_icon": "fa fa-folder-open fa-fw warning",
                                "action_name": "Désarchiver ce cours"}

        messages_dict = {"1": "Vous n'avez aucun cours en favoris.",
                         "2": "Vous n'avez pas encore de cours dans Jalon. Pour ajouter un nouveau cours, cliquez sur le bouton « Créer un cours » ci-dessus.",
                         "3": "Vous n'êtes co-auteur d'aucun cours.",
                         "4": "Vous n'êtes lecteur d'aucun cours.",
                         "5": "Vous n'avez aucun cours archivé."}
        if not courses_list:
            return {"is_courses_list": False,
                    "message": messages_dict[tab]}
        authors_dict = {}
        for course_brain in courses_list:
            if course_brain.getId not in courses_ids_list:
                # if tab not in ["1", "5"]:
                if not tab == "5":
                    # if (member_id not in course_brain.Subject) and (member_id not in course_brain.getArchive):
                    if member_id not in course_brain.getArchive:
                        courses_list_filter.append(
                            self.getCourseData(course_brain, authors_dict, member_id, member_login_time, tab, actions_list))
                else:
                    courses_list_filter.append(
                        self.getCourseData(course_brain, authors_dict, member_id, member_login_time, tab, actions_list))
                courses_ids_list.append(course_brain.getId)
        return {"is_courses_list": True if courses_list_filter else False,
                "message": messages_dict[tab],
                "courses_list":    list(courses_list_filter)}

    def getStudentCoursesList(self, member, tab, folder, is_tab_password):
        # LOG.info("----- getStudentCoursesList -----")
        diploma_list = []
        authors_dict = {}
        portal = self.context.portal_url.getPortalObject()
        portal_catalog = portal.portal_catalog
        portal_jalon_bdd = portal.portal_jalon_bdd

        user_id = member.getId()
        member_login_time = member.getProperty('login_time', None)

        course_authorized_list = []
        if tab == "1":
            user_diploma_list = []
            for diploma_data in portal_jalon_bdd.getInscriptionIND(user_id, "etape"):
                user_diploma_list.append(diploma_data["COD_ELP"])

            if not user_diploma_list:
                # pour les nouveaux étudiant qui n'ont pas encore de diplome ou les étudiants invités par email
                courses_list = []
                search_courses_list = list(portal_catalog.searchResults(portal_type="JalonCours", getRechercheAcces=user_id))
                for course_brain in search_courses_list:
                    course_data = self.getCourseData(course_brain, authors_dict, user_id, member_login_time, tab, [])
                    # course_data["course_access"] = ["Invité"]
                    courses_list.append(course_data)
                    course_authorized_list.append(course_data["course_id"])
                diploma_list.append({"diploma_course_list": courses_list})
                self.request.SESSION.set("course_authorized_list", course_authorized_list)
                # LOG.info(course_authorized_list)
                return {"is_diploma_list": True,
                        "diploma_list":    diploma_list}

            # educational_unity_dict = {}
            educational_unity_list = []
            for user_diploma_id in user_diploma_list:
                user_diploma_data = portal_jalon_bdd.getVersionEtape(user_diploma_id)
                if user_diploma_data:
                    # educational_unity_dict["etape*-*%s" % user_diploma_id] = {"type":    "etape",
                    #                                                          "libelle": user_diploma_data[0]}
                    educational_unity_list.append("etape*-*%s" % user_diploma_id)
                    user_educational_registration_list = portal_jalon_bdd.getInscriptionPedago(user_id, user_diploma_id)
                    if not user_educational_registration_list:
                        user_educational_registration_list = portal_jalon_bdd.getUeEtape(user_diploma_id)
                    for user_educational_registration in user_educational_registration_list:
                        ELP = "*-*".join([user_educational_registration["TYP_ELP"], user_educational_registration["COD_ELP"]])
                        # if ELP not in educational_unity_dict:
                        if ELP not in educational_unity_list:
                            # educational_unity_dict[ELP] = {"type":    user_educational_registration["TYP_ELP"],
                            #                               "libelle": user_educational_registration["LIB_ELP"]}
                            educational_unity_list.append(ELP)
                    educational_unity_list.append(user_id)

                    query = {"query":    educational_unity_list,
                             "operator": "or"}
                    course_list = []
                    course_brain_list = list(portal_catalog.searchResults(portal_type="JalonCours", getRechercheAcces=query))
                    for course_brain in course_brain_list:
                        # course_access_list = []
                        course_data = self.getCourseData(course_brain, authors_dict, user_id, member_login_time, tab, [])
                        """
                        for course_access in course_brain.getListeAcces:
                            if course_access in educational_unity_dict:
                                course_access_list.append("%s : %s" % (educational_unity_dict[course_access]['type'], educational_unity_dict[course_access]['libelle']))
                        if user_id in course_brain.getGroupe:
                            course_access_list.append("Invité")
                        try:
                            if user_id in course_brain.getInscriptionsLibres:
                                course_access_list.append("Inscription par mot de passe")
                        except:
                            pass
                        course_data["course_access"] = course_access_list
                        """
                        course_list.append(course_data)
                        course_authorized_list.append(course_data["course_id"])
                        self.request.SESSION.set("course_authorized_list", course_authorized_list)
                    diploma_list.append({"diploma_name":        user_diploma_data[0],
                                         "diploma_course_list": course_list})
        else:
            course_list = []
            course_brain_list = list(portal_catalog.searchResults(portal_type="JalonCours", getCategorieCours=tab))
            for course_brain in course_brain_list:
                course_data = self.getCourseData(course_brain, authors_dict, user_id, member_login_time, tab, [], is_tab_password)
                course_list.append(course_data)
                if tab != "2":
                    course_authorized_list.append(course_data["course_id"])
            diploma_list.append({"diploma_course_list": course_list})

        self.request.SESSION.set("course_authorized_list", course_authorized_list)
        # LOG.info(course_authorized_list)
        if diploma_list:
            return {"is_diploma_list": True,
                    "diploma_list":    diploma_list}
        else:
            return {"is_diploma_list": False,
                    "message":         "Vous n'êtes inscrit(e) à aucun diplôme."}

    def getCourseData(self, course_brain, authors_dict, member_id, member_login_time, tab, actions_list, is_tab_password=False):
        # LOG.info("----- getCourseData -----")
        course_data = {"course_id":                course_brain.getId,
                       "course_title":             jalon_utils.supprimerMarquageHTML(course_brain.Title),
                       "course_short_title":       jalon_utils.getPlainShortText(course_brain.Title),
                       "course_short_description": jalon_utils.getPlainShortText(course_brain.Description, 210),
                       "course_creator":           course_brain.Creator,
                       "course_is_nouveau":        "fa fa-bell-o fa-fw no-pad" if cmp(course_brain.getDateDerniereActu, member_login_time) > 0 else "",
                       "course_modified":          course_brain.modified,
                       "course_link":              course_brain.getURL,
                       "course_is_etudiants":      "fa fa-check fa-lg no-pad success" if len(course_brain.getRechercheAcces) > 0 else "fa fa-times fa-lg no-pad warning",
                       "course_is_password":       "fa fa-check fa-lg no-pad success" if course_brain.getLibre else "fa fa-times fa-lg no-pad warning",
                       "course_is_public":         "fa fa-check fa-lg no-pad success" if course_brain.getAcces == "Public" else "fa fa-times fa-lg no-pad warning",
                       "course_actions_list":      actions_list}

        if is_tab_password:
            course_data["course_link"] = "%s/check_course_password_form" % course_data["course_link"]()
        course_author_id = course_brain.getAuteurPrincipal
        if not course_author_id:
            course_author_id = course_brain.Creator
        if course_author_id not in authors_dict:
            authors_dict[course_author_id] = jalon_utils.getIndividu(course_author_id, "dict")["fullname"]

        course_data["course_author_id"] = course_author_id
        course_data["course_author_name"] = authors_dict[course_author_id]

        return course_data



## Controller Python Script "evaluate_deposit_box_skill_script"
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=
##title=
##

form = context.REQUEST.form

compEtudiants = dict(context.getCompEtudiants())
for etudiantForm in form["etudiants"]:
    etudiant = dict(etudiantForm)
    etudiant_id = etudiant["id"]
    del etudiant["id"]
    compEtudiants[etudiant_id] = etudiant

context.setCompEtudiants(compEtudiants)
context.REQUEST.RESPONSE.redirect("%s?tab=skills" % context.absolute_url())



## Controller Python Script "modify_favorite_course_script"
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=
##title=
##

tab = context.modifyArchiveCourse(context.REQUEST.form["user_id"], context.REQUEST.form["course_id"])

context.REQUEST.RESPONSE.redirect("%s/mes_cours?tab=2" % context.absolute_url())



##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=
##title=
##
#import OEF_to_OLX

u"""  Script (Python) "export_wims_exercice_script".

Fournit un fichier telechargeable de l'exo WIMS courant, selon le format demandé (QTI, EDX, OEF...)

"""
# context = context
request = context.REQUEST
modele  = context.getModele()

portal = context.aq_parent
authMember = portal.portal_membership.getAuthenticatedMember()

# request n'est pas un veritable dico, donc pas de "in"
if request.has_key("format"):
    file_format = request["format"]
else:
    # Par défaut, on exporte en QTI
    file_format = "QTI"

if request.has_key("version"):
    version = request["version"]
else:
    # Par défaut, on exporte dans la derniere version
    version = "latest"

if file_format == "QTI":
    filename = "%s_WIMS_%s-%s.zip" % (modele, file_format, version)
    zipfile = context.getExoZIP("questionDB.xml", context.getExoXML(file_format, version))
    request.RESPONSE.setHeader('content-type', "application/zip")
    request.RESPONSE.setHeader('content-length', zipfile["length"])
    request.RESPONSE.setHeader('Content-Disposition', 'attachment; filename=%s' % filename)
    # Ici, context doit etre un jalonexercicewims
    return zipfile["data"]

elif file_format == "OEF":
    filename = "%s.oef" % context.getId()
    request.RESPONSE.setHeader('content-type', "text/plain")
    request.RESPONSE.setHeader('Content-Disposition', 'attachment; filename=%s' % filename)
    exo_file = context.getExoOEF(modele, authMember, request)
    if exo_file["request_status"] == "OK":
        return exo_file["code_source"]
    else:
        return exo_file["error_message"]

elif file_format == "FLL":
    filename = "%s.fll" % context.getId()
    request.RESPONSE.setHeader('content-type', "application/xml")
    request.RESPONSE.setHeader('Content-Disposition', 'attachment; filename=%s' % filename)
    # Ici, context doit etre un jalonexercicewims
    return context.getExoXML(file_format, version)

else:
    filename = "%s_WIMS_%s.xml" % (modele, file_format)
    request.RESPONSE.setHeader('content-type', "application/xml")
    request.RESPONSE.setHeader('Content-Disposition', 'attachment; filename=%s' % filename)
    # Ici, context doit etre un jalonexercicewims
    return context.getExoXML(file_format, version)



##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=format,lang
##title=
##
u"""  Script Python download_wims_scores_script (ex "listingTableur_wims").

Fournit un fichier de notes à télécharger (format CSV ou TSV)

"""

request = context.REQUEST

types = {"csv": "application/CSV", "tsv": 'text/tab-separated-values'}
format = "tsv"
lang = "fr"

#request n'est pas un veritable dico, donc pas de "in"
if request.has_key("format") and request["format"] in types:
    format = request["format"]
if request.has_key("lang"):
    lang = request["lang"].split("-")[0]

request.RESPONSE.setHeader('content-type', types[format])
request.RESPONSE.setHeader('Content-Disposition', 'attachment; filename=liste_notes.%s' % format)

# Ici, context doit etre un jaloncourswims (autoeval / examen )
return context.getNotesTableur(format=format, site_lang=lang)



## Controller Python Script "read_course_map_item_script"
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=
##title=Activite edit
##

item_id = context.REQUEST["item_id"]

context.readCourseMapItem(item_id)

if context.REQUEST.HTTP_X_REQUESTED_WITH != 'XMLHttpRequest':
    context.REQUEST.RESPONSE.redirect("%s?mode_etudiant=true" % context.absolute_url())



# Controller Python Script "open_course_map_item_script"
# bind container=container
# bind context=context
# bind namespace=
# bind script=script
# bind subpath=traverse_subpath
# parameters=
# title=
##

"""
    form: {
        id: 'ID_du_li_concerne' pour un chapitre / 'all' pour l'ensemble ,
        open: 'true' = a deplier / 'false' = a replier,
    }

"""
form = context.REQUEST.form

return form # Pour affichage dans la console lors du retour Ajax.



# -*- coding: utf-8 -*-
u"""jalonexportswims : librairie de scripts permettant d'exporter des EXO WIMS dans différents formats."""

import xml.etree.ElementTree as ET
# import HTMLParser

import jalon_utils

from zipfile import ZipFile, ZIP_DEFLATED
import os

from random import shuffle

import string
import lxml

from logging import getLogger
LOG = getLogger('jalonExportsWims')
"""
# Log examples :
LOG.debug('debug message')
# LOG.info('info message')
LOG.warn('warn message')
LOG.error('error message')
LOG.critical('critical message')
"""

alphabet = list(string.ascii_uppercase)

# lxml parser est plus permissif qu'elementTree : grace a l'option "recover" il est capable d'ignorer les balises attributs html non conforms en XML.
parser_lxml = lxml.etree.XMLParser(recover=True)


def getExoTXT(context, format="GIFT", version="latest"):
    """Permet de renvoyer un exo WIMS au format plain text (GIFT).

    # Plus d'infos sur le format GIFT : https://docs.moodle.org/30/en/GIFT_format

    """


def getExoZIP(filename_path, exo_donnees):
    """Fournit les donnees "exo_donnees" compressees au format zip."""
    import tempfile
    fd, path = tempfile.mkstemp('.zipfiletransport')
    os.close(fd)

    zipFile = ZipFile(path, 'w', ZIP_DEFLATED)

    zipFile.writestr(filename_path, exo_donnees)
    zipFile.close()

    fp = open(path, 'rb')
    data = fp.read()
    fp.close()
    return {"length": str(os.stat(path)[6]), "data": data}


def getExoXML(context, formatXML="OLX", version="latest"):
    """Permet de renvoyer un exo WIMS au format XML (QTI, OLX, Moodle, ...).

    # Plus d'infos sur le format OLX (format XML d'EDX) : http://edx-open-learning-xml.readthedocs.org/en/latest/problem-xml/index.html
    # Plus d'infos sur le format IMS QTI : https://webapps.ph.ed.ac.uk/qtiworks
    # Plus d'infos sur le format Moodle XML : https://docs.moodle.org/30/en/Moodle_XML_format
    # Plus d'infos sur le format "Flow Lesson Builder 2" (.fll) : impossible de trouver des infos sur ce format... :/ - http://www.turningtechnologies.eu/downloads/

    """
    """
    Imports supportés par TurningPoint 5.3 :
    # Attention : TurningPoint 5 n'est plus supporté par Turning Techs depuis janvier 2016...
    # Un document Word (.doc,.docx) doit contenir le texte de la question en titre 1 et le texte de la réponse en titre 2.
        Seules les questions à choix multiples peuvent être importées.
        Le type de question peut être modifié après l'importation.
    # Un document QTI peut être importé à partir de :
        * Respondus® (3.5 à 4.0) (fichier zip XML RAA) - Logiciel Windows uniquement, tarif $150.
        * Examview® (5.1 à 7.0) (fichier HTML sans les polices).
    # Un document RTF peut être importé à partir d'Examview® (7.0 - 8.0) (galerie de style: par défaut).
    """
    modele = context.getModele()
    # LOG.info("[getExoXML] modele = %s" % modele)
    portal = context.portal_url.getPortalObject()
    membership_tool = portal.portal_membership

    if not membership_tool.isAnonymousUser():
        demandeur = membership_tool.getAuthenticatedMember()
        """
        try:
            source = str(getattr(self, "%s_%s.xml" % (format, modele)))
        except:
            return ("<error>Désolé, ce modèle ne peut être exporté dans le format %s.</error>" % format)
        tree = ET.ElementTree(root)
        for key in dicoXML.keys():
            source = source.replace("$$%s$$" % key, ET.tostring(dicoXML[key])
        """
        parsed_exo = context.getExoWims(modele, demandeur)
        if parsed_exo["code_source"] is None:
            return "<error>%s</error>" % parsed_exo["error_message"]

        parsed_exo["id"] = context.getId()
        # Parametres communs :
        parsed_exo["titre"] = context.Title().decode("utf-8")

        if formatXML == "OLX":
            # apparement, EDX ne prend pas en compte les attributs du problem. (oct. 2015)
            # attention, en choisissant ""rerandomize": "always"", le bouton "check" disparait lors des tests sur EDX... :/
            exoXML = ET.Element("problem", attrib={"rerandomize": "always", "title": parsed_exo["titre"], "display_name": parsed_exo["titre"]})
            elementXML = ET.SubElement(exoXML, "legend")
            elementXML.text = parsed_exo["titre"]
            exoXML = __qcmsimple_to_olx(exoXML, parsed_exo)

        # Format Moodle XML
        elif formatXML == "Moodle_XML":
            if modele == "qcmsimple":
                exoXML = __qcmsimple_to_moodleXML(parsed_exo)

        # Format QTI
        elif formatXML == "QTI":
            if version == "1.1":
                # Format QTI v1.1
                exoXML = ET.Element("assessmentItem",
                                    attrib={"xmlns"             : "http://www.imsproject.org/xsd/ims_qti_rootv1p1",
                                            "xmlns:xsi"         : "http://www.w3.org/2001/XMLSchema-instance",
                                            "xsi:schemaLocation": "http://www.imsproject.org/xsd/ims_qti_rootv1p1 http://www.imsproject.org/xsd/ims_qti_rootv1p1.xsd",
                                            "identifier"        : parsed_exo["id"],
                                            "title"             : parsed_exo["titre"]})
                if modele == "qcmsimple":
                    exoXML = __qcmsimple_to_qti_11(exoXML, parsed_exo)
            elif version == "1.2.1":
                # Format QTI v1.2.1

                # Plus d'infos sur le format QTI v1.2.1 :
                # http://www.imsglobal.org/question/qtiv1p2/imsqti_litev1p2.html
                # http://www.imsglobal.org/question/qtiv1p2/imsqti_oviewv1p2.html
                # http://www.imsglobal.org/question/qtiv1p2/imsqti_asi_outv1p2.html  "ASI Outcomes Processing"
                # http://www.imsglobal.org/question/qtiv1p2/imsqti_asi_bestv1p2.html "ASI Best Practice & Implementation Guide"
                # http://www.imsglobal.org/question/qtiv1p2/imsqti_asi_bindv1p2.html "ASI XML Binding Specification"
                # XSD :
                # ASI LITE                        https://www.imsglobal.org/sites/default/files/xsd/ims_qtilitev1p2p1.xsd
                # ASI (Assessment, Section, Item) https://www.imsglobal.org/sites/default/files/xsd/ims_qtiasiv1p2p1.xsd
                # RES (Results Reporting)         https://www.imsglobal.org/sites/default/files/xsd/ims_qtiresv1p2p1.xsd

                exoXML = ET.Element("questestinterop",
                                    attrib={"xmlns"             : "http://www.imsglobal.org/xsd/ims_qtiasiv1p2",
                                            "xmlns:xsi"         : "http://www.w3.org/2001/XMLSchema-instance",
                                            "xsi:schemaLocation": "http://www.imsglobal.org/xsd/ims_qtiasiv1p2 http://www.imsglobal.org/xsd/ims_qtiasiv1p2p1.xsd"
                                            })
                elementXML = ET.SubElement(exoXML,
                                           "assessment",
                                           attrib={"ident" : parsed_exo["id"],
                                                   "title" : parsed_exo["titre"]
                                                   })
                elementXML = ET.SubElement(elementXML, "section", attrib={"ident" : parsed_exo["id"]})

                if modele == "qcmsimple":
                    __qcmsimple_to_qti_121(elementXML, parsed_exo)
                elif modele == "qcmsuite":
                    __qcmsuite_to_qti_121(elementXML, parsed_exo)
            else:
                # Format QTI v2.1
                # Plus d'infos sur le format QTI v2.1 :
                # XML Binding : http://www.imsglobal.org/question/qtiv2p1/imsqti_bindv2p1.html

                exoXML = ET.Element("assessmentTest",
                                    attrib={"xmlns"             : "http://www.imsglobal.org/xsd/imsqti_v2p1",
                                            "xmlns:xsi"         : "http://www.w3.org/2001/XMLSchema-instance",
                                            "xsi:schemaLocation": "http://www.imsglobal.org/xsd/imsqti_v2p1 http://www.imsglobal.org/xsd/imsqti_v2p1.xsd",
                                            "identifier"        : parsed_exo["id"],
                                            "title"             : parsed_exo["titre"],
                                            "adaptive"          : "false",
                                            "timeDependent"     : "false"})

                if modele == "qcmsimple":
                    exoXML = __qcmsimple_to_qti_21(exoXML, parsed_exo)
                elif modele == "qcmsuite":
                    exoXML = __qcmsuite_to_qti_21(exoXML, parsed_exo)
                else:
                    exoXML.text = u"Désolé, ce modèle n'est pas pris en charge dans ce format."

        # Format "Flow Lecon", généré par "Lesson Builder" de Turning Technologies
        elif formatXML == "FLL":
            # <TestDefinition xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:xsd="http://www.w3.org/2001/XMLSchema">
            exoXML = ET.Element("TestDefinition",
                                attrib={"xmlns:xsd"         : "http://www.w3.org/2001/XMLSchema",
                                        "xmlns:xsi"         : "http://www.w3.org/2001/XMLSchema-instance"
                                        })

            elementXML = ET.SubElement(exoXML, "name")
            elementXML.text = u"Leçon importée de Jalon"
            elementXML = ET.SubElement(exoXML, "description")
            elementXML.text = u"Leçon importée depuis un exercice WIMS sous Jalon 4.5"
            elementXML = ET.SubElement(exoXML, "author")
            elementXML.text = demandeur.getId()
            # Il faut donner un temps superieur au temps total des questions (unité : s)
            elementXML = ET.SubElement(exoXML, "lessonMaxTime")
            elementXML.text = "30"
            elementXML = ET.SubElement(exoXML, "shuffleQuestions")
            elementXML.text = "true"

            if modele == "qcmsimple":
                exoXML = __qcmsimple_to_FlowXML(exoXML, parsed_exo)

        else:
            exoXML = ET.Element("error")
            exoXML.text = u"Le format '%s' n'est pas pris en charge." % formatXML
        return ET.tostring(exoXML, encoding="utf-8")

    else:
        return "<error>Vous n'avez pas le droit de télécharger ce fichier. Vous devez vous identifier en tant qu'enseignant d'abord.</error>"


def __qcmsimple_to_olx(exoXML, parsed_exo):
    # Modele "QCM Simple" vers OLX:
    # LOG.info("[__qcmsimple_to_olx] parsed_exo = %s" % parsed_exo)
    enonce = jalon_utils.convertHTMLEntitiesToUTF8(parsed_exo["enonce"])
    # enonce = jalon_utils.convertHTMLToXHTML(enonce)
    enonce = '<div class="enonce">%s</div>' % enonce

    exoXML.append(ET.fromstring(enonce, parser=parser_lxml))

    if parsed_exo["options_checkbox"] == 1:
        # Checkbox buttons
        element = "choiceresponse"
        attributs_element = {}
        if parsed_exo["options_checkbox"] == 1:
            # Les modes partiels "halves" et "EDC" existent, mais aucun d'eux ne semble fonctionner avec EDX (oct. 2015)
            if parsed_exo["options_eqweight"] == 1:
                attributs_element["partial_credit"] = "EDC"
            else:
                attributs_element["partial_credit"] = "halves"

        sousElement = "checkboxgroup"
        attributs_sousElement = {"label": parsed_exo["id"],
                                 "answer-pool": parsed_exo["tot"]
                                 }
    else:
        # Radio buttons
        element = "multiplechoiceresponse"
        attributs_element = {"targeted-feedback": ""}
        # partial_credit = "points"

        sousElement = "choicegroup"
        attributs_sousElement = {"label": parsed_exo["id"],
                                 "shuffle": "true",
                                 # EDX est incapable de gérer à la fois "answer-pool" et "shuffle"... :/
                                 # "answer-pool": parsed_exo["tot"]
                                 }

    elementXML = ET.SubElement(exoXML, element, attrib=attributs_element)
    elementXML = ET.SubElement(elementXML,
                               sousElement,
                               attrib=attributs_sousElement)
    liste_bons = parsed_exo["bonnesrep"].decode("utf-8").split("\n")
    liste_id_bons = []
    liste_id_mauvais = []
    nb_rep = 0
    for ligne in liste_bons:
        rep = ligne.strip()
        if rep != "":
            liste_id_bons.append(alphabet[nb_rep])
            nb_rep = nb_rep + 1
            reponse = ET.SubElement(elementXML, "choice", attrib={"correct": "true", "explanation-id": "correct"})
            reponse.text = rep

    liste_mauvais = parsed_exo["mauvaisesrep"].decode("utf-8").split("\n")
    for ligne in liste_mauvais:
        rep = ligne.strip()
        if rep != "":
            liste_id_mauvais.append(alphabet[nb_rep])
            nb_rep = nb_rep + 1
            reponse = ET.SubElement(elementXML, "choice", attrib={"correct": "false", "explanation-id": "incorrect"})
            reponse.text = ligne

    if parsed_exo["options_checkbox"] == 1:
        # en mode "checkbox", on place les feedbacks en compoundhint, a l'interieur du checkboxgroup
        if parsed_exo["feedback_bon"] != "":
            reps = " ".join(liste_id_bons)
            feedback = ET.SubElement(elementXML, "compoundhint", attrib={"value": reps})
            feedback.text = parsed_exo["feedback_bon"].decode("utf-8")
        # Malheuresement, avec cette méthode le feedback de mauvaise réponse n'apparait que si toutes les mauvaises sont cochées.
        # il faudrait trouver le moyen de les afficher si au moins une est cochée.
        if parsed_exo["feedback_mauvais"] != "":
            reps = " ".join(liste_id_mauvais)
            feedback = ET.SubElement(elementXML, "compoundhint", attrib={"value": reps})
            feedback.text = parsed_exo["feedback_mauvais"].decode("utf-8")

    if parsed_exo["hint"] != "" or parsed_exo["help"] != "":
        elementXML = ET.SubElement(exoXML, "demandhint")
        if parsed_exo["hint"] != "":
            aide = ET.SubElement(elementXML, "hint")
            aide.text = parsed_exo["hint"].decode("utf-8")
        if parsed_exo["help"] != "":
            aide = ET.SubElement(elementXML, "hint")
            aide.text = parsed_exo["help"].decode("utf-8")

    if parsed_exo["options_checkbox"] == 0:
        # en mode "radio", on place les feedbacks en feedbacksets
        if parsed_exo["feedback_bon"] != "" or parsed_exo["feedback_mauvais"] != "":
            elementXML = ET.SubElement(exoXML, "targetedfeedbackset")
            if parsed_exo["feedback_mauvais"] != "":
                feedback = ET.SubElement(elementXML, "targetedfeedback", attrib={"explanation-id": "incorrect"})
                chaine = '<div class="detailed-targeted-feedback feedback-hint-incorrect"><div class="hint-label">Incorrect :</div><div class="hint-text">%s</div></div>' % parsed_exo["feedback_mauvais"]
                feedback.append(ET.fromstring(chaine, parser=parser_lxml))
            if parsed_exo["feedback_bon"] != "":
                feedback = ET.SubElement(elementXML, "targetedfeedback", attrib={"explanation-id": "correct"})
                chaine = '<div class="detailed-targeted-feedback feedback-hint-correct"><div class="hint-label">Correct :</div><div class="hint-text">%s</div></div>' % parsed_exo["feedback_bon"]
                feedback.append(ET.fromstring(chaine, parser=parser_lxml))

    if parsed_exo["feedback_general"] != "":
        elementXML = ET.SubElement(exoXML, "solution")
        chaine = '<div class="detailed-solution"><p>Solution</p><p>%s</p></div>' % parsed_exo["feedback_general"]
        elementXML.append(ET.fromstring(chaine, parser=parser_lxml))

    return exoXML


def __qcmsimple_to_qti_11(exoXML, parsed_exo):
    # Modele "QCM Simple" vers QTI 1.1:
    # TODO !!
    ###

    elementXML = ET.SubElement(exoXML,
                               "responseDeclaration",
                               attrib={"identifier":  "RESPONSE",
                                       "cardinality": "multiple",
                                       "baseType":    "identifier"})

    correctResponse = ET.SubElement(elementXML, "correctResponse")

    elementXML = ET.SubElement(exoXML,
                               "outcomeDeclaration",
                               attrib={"identifier":  "SCORE",
                                       "cardinality": "single",
                                       "baseType":    "float"})

    elementXML = ET.SubElement(exoXML, "itemBody")
    choiceInteraction = ET.SubElement(elementXML,
                                      "choiceInteraction",
                                      attrib={"responseIdentifier": "RESPONSE",
                                              "shuffle":            "true",
                                              "maxChoices":         "0"})

    elementXML = ET.SubElement(choiceInteraction, "prompt")
    elementXML.text = parsed_exo["enonce"].decode("utf-8")

    liste_bons = parsed_exo["bonnesrep"].decode("utf-8").split("\n")
    nb_rep = 0
    for ligne in liste_bons:
        nb_rep = nb_rep + 1
        rep_id = "rep_%s" % nb_rep
        elementXML = ET.SubElement(correctResponse, "value")
        elementXML.text = rep_id
        elementXML = ET.SubElement(choiceInteraction,
                                   "simpleChoice",
                                   attrib={"identifier": rep_id,
                                           "fixed":      "false"})
        elementXML.text = ligne

    liste_mauvais = parsed_exo["mauvaisesrep"].decode("utf-8").split("\n")
    for ligne in liste_mauvais:
        nb_rep = nb_rep + 1
        rep_id = "rep_%s" % nb_rep
        elementXML = ET.SubElement(choiceInteraction,
                                   "simpleChoice",
                                   attrib={"identifier": rep_id,
                                           "fixed":      "false"})
        elementXML.text = ligne
    return exoXML


def __qcmsimple_to_qti_121(exoXML, parsed_exo):
    """Modele "QCM Simple" vers QTI 1.2.1."""
    # TODO !!

    ###
    # Pour TurningPoint, le title doit absolument ressembler à ca "parsed_exo["titre"] Question MC #N"
    element_item = ET.SubElement(exoXML, "item",
                                 attrib={"ident" : parsed_exo["id"],
                                         "title" : "%s Question MC #1" % parsed_exo["titre"]})

    element_metadata = ET.SubElement(element_item, "itemmetadata")
    elementXML = ET.SubElement(element_metadata, "qmd_itemtype")
    elementXML.text = "qcmsimple"
    elementXML = ET.SubElement(element_metadata, "qmd_toolvendor")
    elementXML.text = "Jalon @ http://jalon.unice.fr"

    elementXML = ET.SubElement(element_item, "presentation")
    element_flow = ET.SubElement(elementXML, "flow")
    __add_matttext(element_flow, parsed_exo["enonce"].decode("utf-8"), flow_type=None)

    # rcardinality (optional - enumerated list: Single, Multiple, Ordered. Default=Single). Indicates the number of responses expected from the user.
    if parsed_exo["options_checkbox"] == 1:
        rcardinality = "Multiple"
    else:
        rcardinality = "Single"

    elementXML = ET.SubElement(element_flow,
                               "response_lid",
                               attrib={"ident": "%s_RL" % parsed_exo["id"],
                                       "rcardinality": rcardinality,
                                       "rtiming":      "No"})
    element_renderchoice = ET.SubElement(elementXML,
                                         "render_choice",
                                         attrib={"shuffle": "Yes"})

    element_resprocessing = ET.SubElement(element_item, "resprocessing")
    elementXML = ET.SubElement(element_resprocessing, "outcomes")
    elementXML = ET.SubElement(elementXML, "decvar")

    # Vu que TurningPoint 5 est incapable de faire de l'aléatoire, on mélange les propositions...
    rep_items = __randomize_responses_qcmsimple(parsed_exo["bonnesrep"], parsed_exo["mauvaisesrep"])

    for rep_id, rep_item in rep_items:
        elementXML = ET.SubElement(element_renderchoice,
                                   "response_label",
                                   attrib={"ident": rep_id})
        __add_matttext(elementXML, rep_item["data"])

        """ apparament, TT ne supporte qu'un varequal par conditionvar.
        Du coup il faut boucler sur respcondition au lieu de varequal..."""
        element_respcondition = ET.SubElement(element_resprocessing,
                                              "respcondition",
                                              attrib={"title": rep_item["type"],
                                                      "continue": "Yes"})
        elementXML = ET.SubElement(element_respcondition, "conditionvar")
        elementXML = ET.SubElement(elementXML,
                                   "varequal",
                                   attrib={"respident": parsed_exo["id"]})
        elementXML.text = rep_id

        elementXML = ET.SubElement(element_respcondition,
                                   "setvar",
                                   attrib={"action": "Add"})

        elementXML.text = rep_item["value"]
        if rep_item["value"] == "0":
            feedbacktype = "Solution"
        else:
            feedbacktype = "Response"
        elementXML = ET.SubElement(element_respcondition,
                                   "displayfeedback",
                                   attrib={"feedbacktype": feedbacktype,
                                           "linkrefid":    rep_item["type"]})

    # Feedbacks #

    # Feedback bon #
    elementXML = ET.SubElement(element_item,
                               "itemfeedback",
                               attrib={"ident": "Correct"})
    __add_matttext(elementXML, parsed_exo["feedback_bon"].decode("utf-8"))

    # Feedback Mauvais (+Solution) #
    elementXML = ET.SubElement(element_item,
                               "itemfeedback",
                               attrib={"ident": "Incorrect"})
    elementXML = ET.SubElement(elementXML,
                               "solution",
                               attrib={"feedbackstyle": "Complete"})
    elementXML = ET.SubElement(elementXML, "solutionmaterial")
    __add_matttext(elementXML, parsed_exo["feedback_mauvais"].decode("utf-8"))

    """
                                 "tot"             : "5",
                                 "givetrue"        : "2",
                                 "minfalse"        : "0",
                                 "feedback_general": "",
                                 "options"         : "checkbox split",
                                 "accolade"        : "1",
                                 "credits"         : "",
                                 "hint"            : "",
                                 "help"            : "",
    """
    return exoXML


def __add_matttext(elementXML, texte, flow_type="flow_mat"):
    if flow_type is not None:
        elementXML = ET.SubElement(elementXML, flow_type)
    elementXML = ET.SubElement(elementXML, "material")
    elementXML = ET.SubElement(elementXML, "mattext",
                               attrib={"texttype": "text/html"})
    elementXML.text = texte


def __qcmsimple_to_qti_21(exoXML, parsed_exo):
    # Modele "QCM Simple" vers QTI 2.1:

    assessmentItem = ET.SubElement(exoXML,
                                   "assessmentItem",
                                   attrib={"xmlns"             : "http://www.imsglobal.org/xsd/imsqti_v2p1",
                                           "xmlns:xsi"         : "http://www.w3.org/2001/XMLSchema-instance",
                                           "xsi:schemaLocation": "http://www.imsglobal.org/xsd/imsqti_v2p1 http://www.imsglobal.org/xsd/imsqti_v2p1.xsd",
                                           "identifier"        : parsed_exo["id"],
                                           "title"             : parsed_exo["titre"],
                                           "adaptive"          : "false",
                                           "timeDependent"     : "false"})

    if parsed_exo["options_checkbox"] == 1:
        cardinality = "multiple"
    else:
        cardinality = "single"

    elementXML = ET.SubElement(assessmentItem,
                               "responseDeclaration",
                               attrib={"identifier":  "RESPONSE",
                                       "cardinality": cardinality,
                                       "baseType":    "identifier"})

    correctResponse = ET.SubElement(elementXML, "correctResponse")

    elementXML = ET.SubElement(assessmentItem,
                               "outcomeDeclaration",
                               attrib={"identifier":  "SCORE",
                                       "cardinality": "single",
                                       "baseType":    "float"})

    elementXML = ET.SubElement(assessmentItem, "itemBody")
    choiceInteraction = ET.SubElement(elementXML,
                                      "choiceInteraction",
                                      attrib={"responseIdentifier": "RESPONSE",
                                              "shuffle":            "true",
                                              "maxChoices":         "0"})

    elementXML = ET.SubElement(choiceInteraction, "prompt")
    elementXML.text = parsed_exo["enonce"].decode("utf-8")

    liste_bons = parsed_exo["bonnesrep"].decode("utf-8").split("\n")
    nb_rep = 0
    for ligne in liste_bons:
        rep = ligne.strip()
        if rep != "":
            nb_rep = nb_rep + 1
            rep_id = "rep_%s" % nb_rep
            elementXML = ET.SubElement(correctResponse, "value")
            elementXML.text = rep_id
            elementXML = ET.SubElement(choiceInteraction,
                                       "simpleChoice",
                                       attrib={"identifier": rep_id,
                                               "fixed":      "false"})
            elementXML.text = rep

    liste_mauvais = parsed_exo["mauvaisesrep"].decode("utf-8").split("\n")
    for ligne in liste_mauvais:
        rep = ligne.strip()
        if rep != "":
            nb_rep = nb_rep + 1
            rep_id = "rep_%s" % nb_rep
            elementXML = ET.SubElement(choiceInteraction,
                                       "simpleChoice",
                                       attrib={"identifier": rep_id,
                                               "fixed":      "false"})
            elementXML.text = rep

    if parsed_exo["feedback_general"] != "":
        elementXML = ET.SubElement(assessmentItem,
                                   "modalFeedback",
                                   attrib={"outcomeIdentifier": "FEEDBACK",
                                           "identifier": "COMMENT",
                                           "showHide": "show"})
        elementXML.text = parsed_exo["feedback_general"].decode("utf-8")

    # "options"         : "checkbox split",
    # <modalFeedback outcomeIdentifier="FEEDBACK" showHide="show" identifier="correct">correct</modalFeedback>
    # <modalFeedback outcomeIdentifier="FEEDBACK" showHide="show" identifier="incorrect">incorrect</modalFeedback>
    elementXML = ET.SubElement(assessmentItem, "responseProcessing", attrib={"template": "http://www.imsglobal.org/question/qti_v2p1/rptemplates/match_correct"})
    return exoXML


def __qcmsimple_to_moodleXML(parsed_exo):
    """Modele "QCM Simple" vers moodle XML."""
    exoXML = ET.Element("question", attrib={"type": "multichoice"})

    elementXML = ET.SubElement(exoXML, "name")
    elementXML = ET.SubElement(elementXML, "text")
    elementXML.text = parsed_exo["titre"]

    elementXML = ET.SubElement(exoXML, "questiontext", attrib={"format": "html"})
    elementXML = ET.SubElement(elementXML, "text")
    chaine = "<![CDATA[%s]]>" % parsed_exo["enonce"]
    elementXML.append(ET.fromstring(chaine))

    """
    <question type="multichoice">
        <name>
          <text>MULTIPLE CHOICE</text>
        </name>
        <questiontext format="html">
          <text><![CDATA[<p>Couleur du cheval ?</p>]]></text>
        </questiontext>
        <generalfeedback format="html">
          <text><![CDATA[<p>BRAVO !!</p>]]></text>
        </generalfeedback>
        <defaultgrade>1.0000000</defaultgrade>
        <penalty>0.3333333</penalty>
        <hidden>0</hidden>
        <single>true</single>
        <shuffleanswers>true</shuffleanswers>
        <answernumbering>abc</answernumbering>
        <correctfeedback format="html">
          <text>Your answer is correct.</text>
        </correctfeedback>
        <partiallycorrectfeedback format="html">
          <text>Your answer is partially correct.</text>
        </partiallycorrectfeedback>
        <incorrectfeedback format="html">
          <text>Your answer is incorrect.</text>
        </incorrectfeedback>
        <shownumcorrect/>
        <answer fraction="0" format="html">
          <text><![CDATA[<p>Bleu</p>]]></text>
          <feedback format="html">
            <text></text>
          </feedback>
        </answer>
        <answer fraction="100" format="html">
          <text><![CDATA[<p>Blanc</p>]]></text>
          <feedback format="html">
            <text></text>
          </feedback>
        </answer>
    </question>

    """


def __qcmsimple_to_FlowXML(exoXML, parsed_exo):
    """Modele "QCM Simple" vers le format XML Flow Lecon (.fll)."""
    elementXML = ET.SubElement(exoXML, "questions")
    element_question = ET.SubElement(elementXML, "QuestionDefinition")

    elementXML = ET.SubElement(element_question, "qExternalId")
    elementXML.text = "1"

    elementXML = ET.SubElement(element_question, "qInternalId")
    elementXML.text = parsed_exo["id"]

    elementXML = ET.SubElement(element_question, "qname")
    elementXML.text = "<![CDATA[%s]]>" % parsed_exo["titre"].decode("utf-8")

    elementXML = ET.SubElement(element_question, "questionMaxTime")
    elementXML.text = "30"

    elementXML = ET.SubElement(element_question, "useLetters")
    elementXML.text = "true"

    # nombre de points de la question
    elementXML = ET.SubElement(element_question, "grade")
    elementXML.text = "1"

    # Types : {0:"Radio Button", 1:"Checkbox", 5:"vrai/faux", ... ?}
    elementXML = ET.SubElement(element_question, "questionType")

    if parsed_exo["options_checkbox"] == 1:
        elementXML.text = "1"
    else:
        elementXML.text = "0"

    elementXML = ET.SubElement(element_question, "allowPartialCredit")
    if parsed_exo["options_split"] == 1:
        elementXML.text = "true"
    else:
        elementXML.text = "false"

    elementAnswers = ET.SubElement(element_question, "questionAnswers")

    # Vu que Lesson Builder 2 est incapable de faire de l'aléatoire, on mélange les propositions...
    rep_items = __randomize_responses_qcmsimple(parsed_exo["bonnesrep"], parsed_exo["mauvaisesrep"])
    liste_questions = []
    nb_rep = 0
    # for ligne in liste_bons:
    for rep_id, rep_item in rep_items:
        if nb_rep <= len(alphabet):
            elementReponse = ET.SubElement(elementAnswers, "answer")
            elementXML = ET.SubElement(elementReponse, "aId")
            elementXML.text = "%s" % nb_rep
            elementXML = ET.SubElement(elementReponse, "aKey")
            elementXML.text = alphabet[nb_rep]
            # elementXML = ET.SubElement(elementReponse, "aText")
            # elementXML.text = alphabet[nb_rep]
            elementXML = ET.SubElement(elementReponse, "isCorrect")
            if rep_item["value"] == "1":
                elementXML.text = "true"
            else:
                elementXML.text = "false"
            liste_questions.append("<li>%s - %s</li>" % (alphabet[nb_rep], rep_item["data"]))
            nb_rep = nb_rep + 1

    elementXML = ET.SubElement(element_question, "questionContent")

    # On intègre les proposition dans l'enoncé pour les faire apparaitre.
    # Pour le moment, un bug d'affichage dans Flow empeche d'utiliser les puces autres que numériques pour les <ol>...
    # questionContent = "<div class=\"oef_explain\" style=\"font-size:24px;\">%s</div><ol type=\"A\" style=\"list-style-type: upper-alpha;font-size:24px;\">%s</ol>" % (parsed_exo["enonce"].decode("utf-8"), "\n".join(liste_questions))
    questionContent = "<div class=\"oef_explain\" style=\"font-size:24px;\">%s</div><ul style=\"font-size:24px;\">%s</ul>" % (parsed_exo["enonce"].decode("utf-8"), "\n".join(liste_questions))

    elementXML.text = "<![CDATA[%s]]>" % questionContent

    # Il semblerait que questionContentType = 6 correspondrait a du contenu HTML
    elementXML = ET.SubElement(element_question, "questionContentType")
    elementXML.text = "6"
    return exoXML
    # elementXML = ET.SubElement(element_question, "advancedMode")
    # elementXML.text = "false"

    """  Checkbox :
      <questionType>1</questionType>
      <questionMaxTime>30</questionMaxTime>
      <useLetters>true</useLetters>
      <allowChangeAnswer>false</allowChangeAnswer>

      <advancedMode>false</advancedMode>
      <rightAnswerTolerance>0</rightAnswerTolerance>
      <grade>1</grade>
      <penalty>0</penalty>
      <questionAnswers>
        <answer>
          <aId>0</aId>
          <aKey>A</aKey>
          <aPercentValue>50</aPercentValue>
          <aText>A</aText>
          <isCorrect>true</isCorrect>
        </answer>
        <answer>
          <aId>1</aId>
          <aKey>B</aKey>
          <aPercentValue>0</aPercentValue>
          <aText>B</aText>
          <isCorrect>false</isCorrect>
        </answer>
        <answer>
          <aId>2</aId>
          <aKey>C</aKey>
          <aPercentValue>50</aPercentValue>
          <aText>C</aText>
          <isCorrect>true</isCorrect>
        </answer>
        <answer>
          <aId>3</aId>
          <aKey>D</aKey>
          <aPercentValue>0</aPercentValue>
          <aText>D</aText>
          <isCorrect>false</isCorrect>
        </answer>
      </questionAnswers>
    </QuestionDefinition>
    """


def __qcmsuite_to_qti_121(exoXML, parsed_exo):
    u"""Modele "QCM à la suite" vers QTI 1.2.1 (principalement pour TurningPoint 5)."""
    for index, id_question in enumerate(parsed_exo["list_id_questions"]):
        element_item = ET.SubElement(exoXML, "item",
                                     attrib={"ident" : id_question,
                                             "title" : "%s Question MC #%s" % (parsed_exo["titre"], index + 1)})

        element_metadata = ET.SubElement(element_item, "itemmetadata")
        elementXML = ET.SubElement(element_metadata, "qmd_itemtype")
        elementXML.text = "qcmsuite"
        elementXML = ET.SubElement(element_metadata, "qmd_toolvendor")
        elementXML.text = "Jalon @ http://jalon.unice.fr"

        elementXML = ET.SubElement(element_item, "presentation")
        element_flow = ET.SubElement(elementXML, "flow")
        __add_matttext(element_flow, parsed_exo["enonce%s" % index].decode("utf-8"), flow_type=None)

        # rcardinality (optional - enumerated list: Single, Multiple, Ordered. Default=Single). Indicates the number of responses expected from the user.
        if parsed_exo["anstype"] == "checkbox":
            rcardinality = "Multiple"
        else:
            rcardinality = "Single"

        elementXML = ET.SubElement(element_flow,
                                   "response_lid",
                                   attrib={"ident": "%s_RL" % id_question,
                                           "rcardinality": rcardinality,
                                           "rtiming":      "No"})

        element_renderchoice = ET.SubElement(elementXML,
                                             "render_choice",
                                             attrib={"shuffle": "Yes"})

        element_resprocessing = ET.SubElement(element_item, "resprocessing")
        elementXML = ET.SubElement(element_resprocessing, "outcomes")
        elementXML = ET.SubElement(elementXML, "decvar")

        nb_rep = 0

        dico_reponses = {}
        # answers
        liste_reponses = parsed_exo["reponses%s" % index].split("\n")

        for num_ligne, ligne in enumerate(liste_reponses):
            if num_ligne == 0:
                bonnes_reps = ligne.split(",")
            else:
                nb_rep = nb_rep + 1
                rep_id = "rep_%s" % nb_rep
                # LOG.info("[__qcmsuite_to_qti_121] bonnes_reps = %s" % bonnes_reps)
                if str(num_ligne) in bonnes_reps:
                    # LOG.info("[__qcmsuite_to_qti_121] correct == %s" % num_ligne)
                    type_rep = "Correct"
                    value = "1"  # On donne "+1" par bonne réponse.
                else:
                    type_rep = "Incorrect"
                    value = "0"
                dico_reponses[rep_id] = {"type":        type_rep,
                                         "data":        ligne.decode("utf-8"),
                                         "value":       value,
                                         "feedbacktype": "Response"}

        rep_items = dico_reponses.items()
        shuffle(rep_items)

        for rep_id, rep_item in rep_items:
            elementXML = ET.SubElement(element_renderchoice,
                                       "response_label",
                                       attrib={"ident": rep_id})
            __add_matttext(elementXML, rep_item["data"])

            """ apparament, TP5 ne supporte qu'un varequal par conditionvar.
            Du coup il faut boucler sur respcondition au lieu de varequal..."""
            element_respcondition = ET.SubElement(element_resprocessing,
                                                  "respcondition",
                                                  attrib={"title": rep_item["type"],
                                                          "continue": "Yes"})
            elementXML = ET.SubElement(element_respcondition, "conditionvar")
            elementXML = ET.SubElement(elementXML,
                                       "varequal",
                                       attrib={"respident": id_question})
            elementXML.text = rep_id

            elementXML = ET.SubElement(element_respcondition,
                                       "setvar",
                                       attrib={"action": "Add"})

            elementXML.text = rep_item["value"]
            elementXML = ET.SubElement(element_respcondition,
                                       "displayfeedback",
                                       attrib={"feedbacktype": rep_item["feedbacktype"],
                                               "linkrefid":    "general"})

        # Feedbacks #

        elementXML = ET.SubElement(element_item,
                                   "itemfeedback",
                                   attrib={"ident": "general"})
        __add_matttext(elementXML, parsed_exo["feedback%s" % index].decode("utf-8"))

    return exoXML


def __qcmsuite_to_qti_21(exoXML, parsed_exo):
    # Modele "QCM a la Suite" vers QTI 2.1:

    if parsed_exo["anstype"] == "checkbox":
        cardinality = "multiple"
    else:
        cardinality = "single"

    for index, id_question in enumerate(parsed_exo["list_id_questions"]):

        assessmentItem = ET.SubElement(exoXML,
                                       "assessmentItem",
                                       attrib={"identifier"        : id_question,
                                               "title"             : parsed_exo["titre"],
                                               "adaptive"          : "false",
                                               "timeDependent"     : "false"})

        elementXML = ET.SubElement(assessmentItem,
                                   "responseDeclaration",
                                   attrib={"identifier":  "RESPONSE",
                                           "cardinality": cardinality,
                                           "baseType":    "identifier"})

        correctResponse = ET.SubElement(elementXML, "correctResponse")

        elementXML = ET.SubElement(assessmentItem,
                                   "outcomeDeclaration",
                                   attrib={"identifier":  "SCORE",
                                           "cardinality": "single",
                                           "baseType":    "float"})

        elementXML = ET.SubElement(assessmentItem, "itemBody")
        choiceInteraction = ET.SubElement(elementXML,
                                          "choiceInteraction",
                                          attrib={"responseIdentifier": "RESPONSE",
                                                  "shuffle":            "true",
                                                  "maxChoices":         "0"})

        elementXML = ET.SubElement(choiceInteraction, "prompt")
        elementXML.text = parsed_exo["enonce%s" % index].decode("utf-8")

        liste_reponses = parsed_exo["reponses%s" % index].split("\n")
        for num_ligne, ligne in enumerate(liste_reponses):
            if num_ligne == 0:
                bonnes_reps = ligne.split(",")
            else:
                rep = ligne.strip()
                if rep != "":
                    rep_id = "rep_%s" % num_ligne
                    if str(num_ligne) in bonnes_reps:
                        elementXML = ET.SubElement(correctResponse, "value")
                        elementXML.text = rep_id
                    elementXML = ET.SubElement(choiceInteraction,
                                               "simpleChoice",
                                               attrib={"identifier": rep_id,
                                                       "fixed":      "false"})
                    elementXML.text = rep.decode("utf-8")

        if parsed_exo["feedback%s" % index] != "":
            elementXML = ET.SubElement(assessmentItem,
                                       "modalFeedback",
                                       attrib={"outcomeIdentifier": "FEEDBACK",
                                               "identifier": "COMMENT",
                                               "showHide": "show"})
            elementXML.text = parsed_exo["feedback%s" % index].decode("utf-8")
        elementXML = ET.SubElement(assessmentItem, "responseProcessing", attrib={"template": "http://www.imsglobal.org/question/qti_v2p1/rptemplates/match_correct"})
    return exoXML


def __randomize_responses_qcmsimple(liste_bons, liste_mauvais):
    # Renvoit un tuple mélangeant les 2 listes en parametres

    liste_bons = liste_bons.decode("utf-8").split("\n")
    liste_mauvais = liste_mauvais.decode("utf-8").split("\n")

    dico_reponses = {}
    nb_rep = 0
    # Correct answers #
    for ligne in liste_bons:
        rep = ligne.strip()
        if rep != "":
            nb_rep = nb_rep + 1
            rep_id = "rep_%s" % nb_rep

            dico_reponses[rep_id] = {"type": "Correct",
                                     "data": rep,
                                     "value": "1",  # On donne "+1" par bonne réponse.
                                     }
    # Incorrect answers #
    liste_mauvais
    for ligne in liste_mauvais:
        rep = ligne.strip()
        if rep != "":
            nb_rep = nb_rep + 1
            rep_id = "rep_%s" % nb_rep
            dico_reponses[rep_id] = {"type": "Incorrect",
                                     "data": rep,
                                     "value": "0",  # On donne "0" par mauvaise réponse.
                                     }

    # Vu que TurningPoint 5 est incapable de faire de l'aléatoire, on mélange les propositions...
    rep_items = dico_reponses.items()
    shuffle(rep_items)
    return rep_items



# -*- coding: utf-8 -*-
"""
This module contains the tool of jalon.content
"""
import os
from setuptools import setup, find_packages


def read(*rnames):
    return open(os.path.join(os.path.dirname(__file__), *rnames)).read()

version = '3.1.1'

long_description = (
    read('README.txt')
    + '\n' +
    'Change history\n'
    '**************\n'
    + '\n' +
    read(os.path.join("docs", 'CHANGES.txt'))
    + '\n' +
    'Detailed Documentation\n'
    '**********************\n'
    + '\n' +
    read('jalon', 'content', 'README.txt')
    + '\n' +
    'Contributors\n'
    '************\n'
    + '\n' +
    read(os.path.join("docs", 'CONTRIBUTORS.txt'))
    + '\n' +
    'Download\n'
    '********\n')

tests_require = ['zope.testing']

setup(name='jalon.content',
      version=version,
      description="",
      long_description=long_description,
      # Get more strings from
      # http://pypi.python.org/pypi?%3Aaction=list_classifiers
      classifiers=['Framework :: Plone',
                   'Intended Audience :: Developers',
                   'License :: OSI Approved :: GNU General Public License (GPL)',
                   ],
      keywords='',
      author='Bordonado Christophe, Jerome Navarro, Olivier Bado - Université Nice Sophia Antipolis (uns) Service TICE / Jean Bado - Université Versaille Saint-Quentin(UVSQ)',
      author_email='tice@unice.fr',
      url='http://unice.fr',
      license='CECILL',
      packages=find_packages(exclude=['ez_setup']),
      namespace_packages=['jalon', ],
      include_package_data=True,
      zip_safe=False,
      install_requires=['setuptools',
                        # -*- Extra requirements: -*-
                        ],
      tests_require=tests_require,
      extras_require=dict(tests=tests_require),
      test_suite='jalon.content.tests.test_docs.test_suite',
      entry_points="""
      # -*- entry_points -*-
      [z3c.autoinclude.plugin]
      target = plone
      """,
      setup_requires=["PasteScript"],
      paster_plugins=["ZopeSkel"],
      )



# -*- coding: utf-8 -*-
"""View scripts associated to my_wims_exercices_view.pt template."""

from zope.component import getMultiAdapter
from my_space_view import MySpaceView

from jalon.content import contentMessageFactory as _

from logging import getLogger
LOG = getLogger('[MyWimsExercicesView]')


class MyWimsExercicesView(MySpaceView):
    """Class View du fichier my_wims_exercices_view.pt."""

    def __init__(self, context, request):
        """Initialize th view class."""
        # LOG.info("----- Init -----")
        MySpaceView.__init__(self, context, request)
        self.context = context
        self.request = request

    def getBreadcrumbs(self):
        """Fournit le fil d'ariane de la vue."""
        return [{"title": _(u"Mes ressources"),
                 "icon":  "fa fa-folder-open",
                 "link":  self.context.aq_parent.absolute_url()},
                {"title": _(u"Mes exercices WIMS"),
                 "icon":  "fa fa-random",
                 "link":  self.context.absolute_url()}]

    def getMyWimsExercicesView(self, user):
        """Fournit la vue "Mes Exercices"."""
        # LOG.info("----- getMyWimsExercicesView -----")
        portal_state = getMultiAdapter((self.context, self.request), name=u'plone_portal_state')
        portal = portal_state.portal()

        user_id = user.getId()
        folder = getattr(portal.Members, user_id).Wims

        selected_tags_list = folder.getSelectedTags().split(",")

        tags = self.getTags(folder, selected_tags_list)
        tags_dict = tags["tags_dict"]
        tags_list = tags["tags_list"]

        portal_wims = getattr(portal, "portal_wims", None)
        my_wims_exercices_list = self.getMyWimsExercicesList(folder, selected_tags_list, user_id, portal_wims)

        one_and_selected_tag = self.getOneAndSelectedTag(my_wims_exercices_list, selected_tags_list, tags_dict)

        if selected_tags_list == ["last"]:
            # dans ce cas, my_wims_exercices_list est un object dont la fonction "len" est deprecated.
            nb_display_items = my_wims_exercices_list.length
        else:
            # dans ce cas, my_wims_exercices_list est un 'LazyMap' object, qui n'a pas d'attribut 'length'
            nb_display_items = len(my_wims_exercices_list)

        nb_items = len(folder.objectIds())

        wims_exercice_model_list = folder.getModelesWims()

        return {"tags_list":                tags_list,
                "is_no_items":              one_and_selected_tag["is_no_items"],
                "is_one_tag":               one_and_selected_tag["is_one_tag"],
                "one_tag":                  one_and_selected_tag["one_tag"],
                "is_selected_tags":         one_and_selected_tag["is_selected_tags"],
                "my_items_list":            my_wims_exercices_list,
                "nb_display_items":         nb_display_items,
                "nb_items":                 nb_items,
                "folder_path":              "/".join(folder.getPhysicalPath()),
                "folder_link":              folder.absolute_url(),
                "wims_exercice_model_list": wims_exercice_model_list}

    def getMyWimsExercicesList(self, folder, selected_tags_list, user_id, portal_wims):
        """Fournit la liste des exercices WIMS de user_id."""
        # LOG.info("----- getMyWimsExercicesList -----")
        # LOG.info("selected_tags_list = ##%s##" % selected_tags_list)
        member_wims_class = folder.getComplement()
        if not member_wims_class:
            self.createWimsClass(folder, user_id, portal_wims)
        elif selected_tags_list == [""]:
            # On ne met à jour la liste des exos que si aucune étiquette est selectionnée.
            self.updateJalonExercicesWims(folder, user_id, member_wims_class, portal_wims)
        content_filter = {"portal_type": ["JalonExerciceWims"]}
        return self.getItemsList(folder, selected_tags_list, content_filter)

    def createWimsClass(self, folder, user_id, portal_wims):
        u"""Crée le groupement de classes WIMS pour un nouvel utilisateur si il n'existe pas déjà."""
        # LOG.info("----- createWimsClass -----")
        # 1er  cas : Aucune classe n'existe pour cet utilisateur
        member_properties = self.context.getIndividu(user_id, "dict")
        member_email = member_properties["email"]
        member_fullname = member_properties["fullname"]
        # Création du groupement de classes
        groupement_wims_class = portal_wims.creerClasse({"authMember": user_id,
                                                         "fullname":   member_fullname,
                                                         "auth_email": member_email,
                                                         "type":       "2",
                                                         "qclass":     ""})
        # Création de la 1ere classe dans le groupement (elle contiendra tous les exos)
        if groupement_wims_class["status"] == "OK":
            wims_class_id = portal_wims.creerClasse({"authMember":   user_id,
                                                     "fullname":     member_fullname,
                                                     "auth_email":   member_email,
                                                     "type":         "0",
                                                     "titre_classe": "Mes exercices",
                                                     "qclass":       groupement_wims_class["class_id"]})
            if wims_class_id:
                folder.complement = str(groupement_wims_class["class_id"])
                return {"status": "OK", "member_wims_class": folder.complement}
        else:
            # print "*****    Mauvais parametrage de votre connexion WIMS  *****"
            # print "[my_wims_exercices.py/createWimsClass] Creation du groupement impossible"
            # print " Reponse WIMS : %s" % groupement
            # print "*****                                                 *****"
            return {"status": "ERROR", "message": "wims_bad_conf"}

    def updateJalonExercicesWims(self, folder, user_id, member_wims_class, portal_wims):
        u"""Met a jour la liste des exos de l'utilisateur user_id."""
        # LOG.info("----- updateJalonExercicesWims -----")
        # L'utilisateur courant dispose bien d'une classe. on liste ses exercices.
        # print "Classe %s" % self.getComplement()
        exercices = portal_wims.getExercicesWims({"authMember": user_id,
                                                  "qclass":     "%s_1" % member_wims_class,
                                                  "jalon_URL":   folder.absolute_url()})
        if exercices["status"] == "ERROR":
            # en cas d'indisponibilite, le code retour de WIMS donne un type "HTTPError"
            if "type" in exercices:
                return {"status": "ERROR", "message": "wims_unavailable"}
            else:
                return {"status": "ERROR", "message": "wims_bad_conf"}
        # except:
        #   mail_body = "*****    WIMS indisponible ou Mauvais parametrage de La connexion WIMS  *****\n"
        #   mail_body += "[my_wims_exercices.py/updateJalonExercicesWims] getExercicesWims\n"
        # mail_body += "#2e  cas : l'utilisateur courant dispose deja d'une classe. on liste ses exercices.\n\n"
        #   mail_body += " authMember : %s \n" % authMember
        #   mail_body += " qclass : %s_1 \n" % self.getComplement()
        #   mail_body += "*****                                                                   *****\n"
        #   print mail_body
        #   mail_erreur["message"] = mail_body
        #   self.envoyerMailErreur(mail_erreur)
        #   Si getExercicesWIMS plante, c'est :
        #   soit une mauvaise configuration de WIMS  par l'admin (elle a du etre changee entre temps, puisqu'il dispose d'une classe ici)
        #   soit que wims est actuellement indisponible. (cas un peu plus probable que le 1er)
        #   return {"erreur" : "wims_unavailable" }

        # Ici il faudrait pouvoir ne lister que les exos qui ne sont pas des groupes
        jalon_wims_exercices = folder.objectIds()
        jalon_wims_exercices = [elem for elem in jalon_wims_exercices if not(elem.startswith("groupe-") or elem.startswith("externe-"))]
        # jalon_wims_exercices = [elem for elem in jalon_wims_exercices if elem.startswith("groupe-")]
        # return {"erreur": "wims_unavailable"}

        if "exocount" in exercices:
            if exercices["exocount"] == 0:
                wims_exercices = []
            else:
                wims_exercices = exercices["exotitlelist"]
            if len(jalon_wims_exercices) < len(wims_exercices):
                # Il y a plus d'exos sur WIMS (%s) que sur Jalon (%s) " % (len(wims_exercices), len(jalon_wims_exercices)))
                model_list = portal_wims.getWimsProperty("modele_wims")
                # On recupere les exos de wims pour les créer sur jalon
                for wims_exercice in wims_exercices:
                    wims_exercice_check = False
                    for exo_jalon in jalon_wims_exercices:
                        if wims_exercice["id"] == exo_jalon:
                            wims_exercice_check = True
                    if not wims_exercice_check:
                        model = wims_exercice["id"].split("-")[0]
                        if model not in model_list:
                            model = "exercicelibre"
                        # LOG.info("CREATION de l'exercice %s sur Jalon " % wims_exercice["id"])
                        object_id = folder.invokeFactory(type_name='JalonExerciceWims', id=wims_exercice["id"])
                        object_created = getattr(folder, object_id)
                        object_created.setProperties({"Title":  wims_exercice["title"],
                                                      "Modele": model})
        else:
            # *****serveur WIMS indisponible ou mauvaise configuration de l'acces WIMS"
            # Si WIMS est indisponible, on ignore simplement sa liste d'exercices et on affiche celle de Jalon uniquement.
            LOG.info("*****    Mauvais parametrage de votre connexion WIMS  *****")
            LOG.info("[jalonfolder.py] getExercicesWims : %s" % exercices)
            LOG.info("*****                                                *****")
            return {"erreur": "wims_unavailable"}



## Script (Python) "delete_all_course_announce_script"
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=
##title=
##

context.deleteAllAnnounce()

context.REQUEST.RESPONSE.redirect(context.absolute_url())



# -*- coding: utf-8 -*-
""" Jalon Properties. """

from zope.interface import implements

from OFS.SimpleItem import SimpleItem
from persistent.dict import PersistentDict
from Products.CMFCore.utils import getToolByName

from twython import Twython, TwythonError

from ..browser.interfaces import IJalonProperties
from jalon.content import contentMessageFactory as _
from jalon.content.content import jalon_utils

from DateTime import DateTime

import json
import copy

# Messages de debug :
from logging import getLogger
LOG = getLogger('[JalonProperties]')


class JalonProperties(SimpleItem):

    """ Jalon Properties Class. """

    implements(IJalonProperties)
    #categories de cours
    _categories = {}

    #gestion podcasts
    _activerPodcasts = False
    _uploadPodcasts = ""
    _dnsPodcasts = ""
    _dicoUsersPodcast = {}
    _dicoLibCatITunesU = {"102":    "Art & Architecture",
                          "102100": "Architecture",
                          "102102": "Art History",
                          "102109": "Culinary Arts",
                          "102103": "Dance",
                          "102105": "Design",
                          "102111": "Fashion",
                          "102104": "Film",
                          "102106": "Interior Design",
                          "102112": "Media Arts",
                          "102107": "Music",
                          "102113": "Photography",
                          "102108": "Theater",
                          "102110": "Visual Art",
                          "100":    "Business",
                          "100100": "Economics",
                          "100107": "Entrepreneurship",
                          "100101": "Finance",
                          "100102": "Hospitality",
                          "100103": "Management",
                          "100104": "Marketing",
                          "100105": "Personal Finance",
                          "100106": "Real Estate",
                          "113":    "Communications & Media",
                          "113101": "Broadcasting",
                          "113102": "Digital Media",
                          "113103": "Journalism",
                          "113104": "Photojournalism",
                          "113105": "Print",
                          "113106": "Speech",
                          "113107": "Writing",
                          "101":    "Engineering",
                          "101100": "Chemical & Petroleum Engineering",
                          "101101": "Civil Engineering",
                          "101102": "Computer Science",
                          "101103": "Electrical Engineering",
                          "101104": "Environmental Engineering",
                          "101105": "Mechanical Engineering",
                          "103":    "Health & Medicine",
                          "103100": "Anatomy & Physiology",
                          "103101": "Behavioral Science",
                          "103102": "Dentistry",
                          "103103": "Diet & Nutrition",
                          "103104": "Emergency Medicine",
                          "103105": "Genetics",
                          "103106": "Gerontology",
                          "103112": "Global Health",
                          "103107": "Health & Exercise Science",
                          "103108": "Immunology",
                          "103109": "Neuroscience",
                          "103114": "Nursing",
                          "103110": "Pharmacology & Toxicology",
                          "103111": "Psychiatry",
                          "103113": "Radiology",
                          "104":    "History",
                          "104104": "African History",
                          "104100": "Ancient History",
                          "104105": "Asia-Pacific History",
                          "104106": "European History",
                          "104101": "Medieval History",
                          "104107": "Middle Eastern History",
                          "104102": "Military History",
                          "104103": "Modern History",
                          "104108": "North American History",
                          "104109": "South American History",
                          "106":    "Language",
                          "106100": "African Languages",
                          "106101": "Ancient Languages",
                          "106114": "Arabic",
                          "106113": "Chinese",
                          "106104": "English",
                          "106106": "French",
                          "106107": "German",
                          "106115": "Hebrew",
                          "106116": "Hindi",
                          "106117": "Indigenous Languages",
                          "106108": "Italian",
                          "106118": "Japanese",
                          "106119": "Korean",
                          "106109": "Linguistics",
                          "106120": "Other Languages",
                          "106121": "Portuguese",
                          "106122": "Russian",
                          "106111": "Spanish",
                          "106112": "Speech Pathology",
                          "116":    "Law & Politics",
                          "116100": "Foreign Policy & International Relations",
                          "116101": "Law",
                          "116102": "Local Governments",
                          "116103": "National Governments",
                          "116104": "Political Science",
                          "116105": "Public Administration",
                          "116106": "World Affairs",
                          "107":    "Literature",
                          "107100": "Anthologies",
                          "107101": "Biography",
                          "107102": "Classics",
                          "107106": "Comparative Literature",
                          "107104": "Fiction",
                          "107103": "Literary Criticism",
                          "107105": "Poetry",
                          "108":    "Mathematics",
                          "108100": "Advanced Mathematics",
                          "108101": "Algebra",
                          "108102": "Arithmetic",
                          "108103": "Calculus",
                          "108104": "Geometry",
                          "108105": "Statistics",
                          "114":    "Philosophy",
                          "114100": "Aesthetics",
                          "114101": "Epistemology",
                          "114102": "Ethics",
                          "114105": "Logic",
                          "114103": "Metaphysics",
                          "114106": "Philosophy of Language",
                          "114107": "Philosophy of Religion",
                          "114104": "Political Philosophy",
                          "110":    "Psychology & Social Science",
                          "110107": "Anthropology",
                          "110106": "Archaeology",
                          "110103": "Psychology",
                          "110104": "Social Welfare",
                          "110105": "Sociology",
                          "115":    "Religion & Spirituality",
                          "115100": "Buddhism",
                          "115101": "Christianity",
                          "115102": "Comparative Religion",
                          "115103": "Hinduism",
                          "115104": "Islam",
                          "115105": "Judaism",
                          "115106": "Other Religions",
                          "115107": "Spirituality",
                          "109":    "Science",
                          "109100": "Agriculture",
                          "109101": "Astronomy",
                          "109102": "Atmosphere",
                          "109103": "Biology",
                          "109104": "Chemistry",
                          "109105": "Ecology",
                          "109109": "Environment",
                          "109106": "Geography",
                          "109107": "Geology",
                          "109108": "Physics",
                          "111":    "Society",
                          "111107": "African Studies",
                          "111108": "American Studies",
                          "111101": "Asia Pacific Studies",
                          "111109": "Cross-cultural Studies",
                          "111102": "European Studies",
                          "111110": "Immigration & Emigration",
                          "111103": "Indigenous Studies",
                          "111104": "Latin & Caribbean Studies",
                          "111105": "Middle Eastern Studies",
                          "111111": "Race & Ethnicity Studies",
                          "111112": "Sexuality Studies",
                          "111106": "Women's Studies",
                          "112":    "Teaching & Learning",
                          "112100": "Curriculum & Teaching",
                          "112101": "Educational Leadership",
                          "112106": "Educational Technology",
                          "112102": "Family & Childcare",
                          "112107": "Information/Library Science",
                          "112103": "Learning Resources",
                          "112104": "Psychology & Research",
                          "112105": "Special Education"}
    _catITunesU = {"102":  ["102100", "102102", "102109", "102103", "102105", "102111", "102104", "102106", "102112", "102107", "102113", "102108", "102110"],
                   "100": ["100100", "100107", "101101", "100102", "100103", "100104", "100105", "100106"],
                   "113": ["113101", "113102", "113103", "113104", "113105", "113106", "113107"],
                   "101": ["101100", "101101", "101102", "101103", "101104", "101105"],
                   "103": ["103100", "103101", "103102", "103103", "103104", "103105", "103106", "103112", "103107", "103108", "103109", "103114", "103110", "103111", "103113"],
                   "104": ["104104", "104100", "104105", "104106", "104101", "104107", "104102", "104103", "104108", "104109"],
                   "106": ["106100", "106101", "106114", "106113", "106104", "106106", "106107", "106115", "106116", "106117", "106108", "106118", "106119", "106109", "106120", "106121", "106122", "106111", "106112"],
                   "116": ["116100", "116101", "116102", "116103", "116104", "116105", "116106"],
                   "107": ["107100", "107101", "107102", "107106", "107104", "107103", "107105"],
                   "108": ["108100", "108101", "108102", "108103", "108104", "108105"],
                   "114": ["114100", "114101", "114102", "114105", "114103", "114106", "114107", "114104"],
                   "110": ["110107", "110106", "110103", "110104", "110105"],
                   "115": ["115100", "115101", "115102", "115103", "115104", "115105", "115106", "115107"],
                   "109": ["109100", "109101", "109102", "109103", "109104", "109105", "109109", "109106", "109107", "109108"],
                   "111": ["111107", "111108", "111101", "111109", "111102", "111110", "111103", "111104", "111105", "111111", "111112", "111106"],
                   "112": ["112100", "112101", "112106", "112102", "112107", "112103", "112104", "112105"]}
    _ordreCatiTunesU = ["102", "100", "113", "101", "103", "104", "106", "116", "107", "108", "114", "110", "115", "109", "111", "112"]

    # Gestion de la connexion
    _activer_cas = 0
    _serviceUrl = ""
    _casServerUrlPrefix = ""
    _activer_creationcompte = 0

    # Gestion du bloc informations
    _activer_etablissement = 1
    _etablissement = "Université Nice Sophia antipolis (UNS)"
    _lien_etablissement = "http://unice.fr"
    _activer_lien_sesame = 1
    _lien_sesame = "http://ent.unice.fr"
    _activer_lien_contact = 1
    _lien_contact = "http://unice.fr/contact-info"
    _activer_lien_mention = 1
    _lien_mention = "http://unice.fr/mentions-legales"
    _activer_lien_credit = 1
    _lien_credit = "http://unice.fr/mentions-legales"

    # Gestion du bloc "Didacticiels"
    _activer_aide = 1
    _lien_aide = "http://jalon.unice.fr/public/xau995"
    _activer_aide_plan = 1
    _lien_aide_plan = '<div class="flex-video"><iframe src="//www.youtube.com/embed/Z10V-BMXuec?list=PLV8G_tL0jD2LmZ79s0D-4EkeCVgQTvWfo" frameborder="0" allowfullscreen></iframe></div>\n<p>Cette vidéo disparaîtra automatiquement quand vous aurez ajouté des éléments à votre cours.</p>'
    _activer_guide_anti_spam = "1"
    _message_guide_anti_spam = """<h3>Pour être sûr(e) de recevoir les courriels de vos enseignants</h3>
<p>Si vous avez fait le choix de rediriger votre courriel @etu.unice.fr vers une autre adresse, vous devez effectuer les réglages permettant d'autoriser le serveur jalon.unice.fr et l'expéditeur noreply@jalon.unice.fr à vous envoyer des courriels et à ne pas les considérer comme du spam.</p>
<dl>
    <dt>Google Mail :</dt>
    <dd>ajoutez noreply@jalon.unice.fr dans vos contacts gmail afin que les messages de cet expéditeur ne soient plus considérés comme du spam.</dd>
    <dt>Hotmail :</dt>
    <dd>rendez-vous dans la rubrique filtre et ajoutez @jalon.unice.fr dans la liste des serveurs expéditeurs autorisés.</dd>
    <dt>Autres :</dt>
    <dd>consultez l'aide de votre webmail à la rubrique spam…</dd>
</dl>"""

    # Gestion du bloc "Message"
    _activer_message_general = 0
    _message_general = ""
    _activer_bie = 0
    _bie_message = ""
    _activer_message_enseignant = 0
    _message_enseignant = ""

    # Gestion du bloc "Courriels"
    _activer_erreur = 0
    _activer_email_erreur = 0
    _adresse_email_erreur = ""
    _activer_liste_diffusion = 0
    _type_diffusion = "etape,ue,uel,groupe"
    _format_diffusion = ""

    # Gestion des bulles de "Mon Espace"
    _activer_fichiers = 1
    _activer_presentations_sonorisees = 0
    _activer_exercices_wims = 0
    _activer_liens = 1
    _activer_liens_catalogue_bu = 0
    _activer_tags_catalogue_bu = 0
    _activer_lille1pod = 0
    _activer_termes_glossaire = 1
    _activer_webconferences = 0
    _activer_lien_intracursus = 0
    _lien_intracursus = "http://intracursus.unice.fr"
    _activer_lien_assistance = 1
    _lien_assitance = "https://sourcesup.renater.fr/forum/forum.php?thread_id=3460&forum_id=2232&group_id=832"
    _activer_video = 0
    _url_video = ""
    _activer_vod = 0

    # Gestion du bloc "Réaseaux sociaux"
    _COMPTE_TWITTER = ""
    _APP_KEY = ""
    _APP_SECRET = ""
    _OAUTH_TOKEN = ""
    _OAUTH_TOKEN_SECRET = ""

    # Gestion de la récupération des données utilisateurs
    _activer_ldap = 0
    _base_ldap = ""
    _schema_ldap = ""
    _fiche_ldap = ""
    _activer_trombinoscope = 0
    _activer_stockage_photo = 0
    _lien_trombinoscope = "http://tice-trombino.unice.fr"

    # Gestion statistique Google Analytics
    _activer_ga = 0
    _ga_id_account = ""
    _ga_id_domain = ""
    _ga_cryptage = ""

    # Gestion de la partie de maintenance
    _annoncer_maintenance = ""
    _activer_maintenance = ""
    _date_debut_maintenance = ""
    _date_fin_maintenance = ""
    _annoncer_vider_cache = ""
    _url_news_maintenance = ""

    def __init__(self, *args, **kwargs):
        super(JalonProperties, self).__init__(*args, **kwargs)
        self._categories = {1: {"title": _(u"Mes formations"), "users": ['all']},
                            2: {"title": _(u"Inscription par mot de passe"), "users": ['all']}}

    #-------------#
    # utilitaires #
    #-------------#
    def getJalonProperty(self, key):
        return getattr(self, "_%s" % key)

    def test(self, condition, valeurVrai, valeurFaux):
        return jalon_utils.test(condition, valeurVrai, valeurFaux)

    def getShortText(self, text, limit=75):
        return jalon_utils.getShortText(text, limit)

    def getJalonPhoto(self, user_id):
        if self._activer_stockage_photo:
            portal = self.portal_url.getPortalObject()
            return "%s/etudiants/%s" % (portal.absolute_url(), user_id)
        else:
            return "%s/%s.jpg" % (self._lien_trombinoscope, user_id)

    def getTopBarMenuLeft(self, user_role):
        # LOG.info("----- getTopBar -----")
        menu_left = []
        is_personnel = user_role in ["Personnel", "Secretaire", "Manager"]

        sub_menu_mes_cours = []
        sub_menu_mes_cours_class = ""
        if user_role in ["Etudiant", "EtudiantJalon"]:
            jalon_categories = dict(self.getCategorie())
            liste_id_categorie = jalon_categories.keys()
            liste_id_categorie.sort()

            for id_categorie in liste_id_categorie:
                sub_menu_mes_cours.append({"id":      "cat%s" % id_categorie,
                                           "icon":    "fa fa-book",
                                           "title":   jalon_categories[id_categorie]['title'],
                                           "link":    "mes_cours?categorie=%s" % id_categorie})
            if sub_menu_mes_cours:
                sub_menu_mes_cours_class = "has-dropdown not-click"

        menu_left.append({"id":       "mes-cours",
                          "class":    sub_menu_mes_cours_class,
                          "icon":     "fa fa-university",
                          "title":    _(u"Mes cours"),
                          "link":     "mes_cours",
                          "sub_menu": sub_menu_mes_cours})

        menu_my_space = None
        if is_personnel:
            menu_my_space = {"id":        "mes_ressources",
                             "class":     "has-dropdown not-click",
                             "icon":      "fa fa-folder-open",
                             "title":     _(u"Mes ressources"),
                             "link":      "mes_ressources",
                             "sub_menu":  []}

            for sub_menu in self.getGridMonEspaceNew():
                if sub_menu["activated"]:
                    menu_my_space["sub_menu"].append(sub_menu)
            menu_left.append(menu_my_space)

            menu_left.append({"id":       "mes_etudiants",
                              "class":    "",
                              "icon":     "fa fa-users",
                              "title":    _(u"Mes étudiants"),
                              "link":     "mes_etudiants",
                              "sub_menu": []})

            menu_left.append({"id":       "tools",
                              "class":    "",
                              "icon":     "fa fa-wrench",
                              "title":    _(u"Outils"),
                              "link":     "tools_page",
                              "sub_menu": []})

        if user_role == "Manager":
            menu_left.append({"id":       "gestion_pedagogique",
                              "class":    "has-dropdown not-click",
                              "icon":    "fa fa-database",
                              "title":    _(u"Gestion pédagogique"),
                              "link":     "portal_jalon_bdd/@@jalon-bdd",
                              "sub_menu": [{"id":    "gestion_bdd",
                                            "icon":  "fa fa-list",
                                            "title": _(u"Offre de formations"),
                                            "link":  "portal_jalon_bdd/@@jalon-bdd?gestion=gestion_bdd"},
                                           {"id":    "gestion_utilisateurs",
                                            "icon":  "fa fa-users",
                                            "title": _(u"Utilsateurs"),
                                            "link":  "portal_jalon_bdd/@@jalon-bdd?gestion=gestion_utilisateurs"},
                                           {"id":    "gestion_connexion_bdd",
                                            "icon": "fa fa-key",
                                            "title": _(u"Propriétés"),
                                            "link":  "portal_jalon_bdd/@@jalon-bdd?gestion=gestion_connexion_bdd"}]})

            menu_left.append({"id":       "configuration",
                              "class":    "has-dropdown not-click",
                              "icon":     "fa fa-cogs",
                              "title":    _(u"Configuration"),
                              "link":     "portal_jalon_properties/@@jalon_properties",
                              "sub_menu": [{"id":    "gestion_connexion",
                                            "icon":  "fa fa-key",
                                            "title": _(u"Connexion à Jalon"),
                                            "link":  "portal_jalon_properties/gestion_connexion"},
                                           {"id":    "gestion_mes_ressources",
                                            "icon":  "fa fa-folder-open",
                                            "title": _(u"Gestion \"Mes ressources\""),
                                            "link":  "portal_jalon_properties/gestion_mes_ressources"},
                                           {"id":    "gestion_mes_cours",
                                            "icon":  "fa fa-university",
                                            "title": _(u"Gestion des cours"),
                                            "link":  "portal_jalon_properties/gestion_mes_cours"},
                                           {"id":    "gestion_infos",
                                            "icon":  "fa fa-external-link-square",
                                            "title": _(u"Liens d'informations"),
                                            "link":  "portal_jalon_properties/gestion_infos"},
                                           {"id":    "gestion_didacticiels",
                                            "icon":  "fa fa-life-ring",
                                            "title": _(u"Didacticiels"),
                                            "link":  "portal_jalon_properties/gestion_didacticiels"},
                                           {"id":    "gestion_messages",
                                            "icon":  "fa fa-bullhorn",
                                            "title": _(u"Diffusion de messages"),
                                            "link":  "portal_jalon_properties/gestion_messages"},
                                           {"id":    "gestion_email",
                                            "icon":  "fa fa-envelope-o",
                                            "title": _(u"Courriels"),
                                            "link":  "portal_jalon_properties/gestion_email"},
                                           {"id":     "gestion_donnees_utilisateurs",
                                            "icon":  "fa fa-users",
                                            "title": _(u"Données utilisateurs"),
                                            "link":  "portal_jalon_properties/gestion_donnees_utilisateurs"},
                                           {"id":     "gestion_ga",
                                            "icon":  "fa fa-line-chart",
                                            "title": _(u"Google Analytics"),
                                            "link":  "portal_jalon_properties/gestion_ga"},
                                           {"id":     "gestion_maintenance",
                                            "icon": "fa fa-umbrella",
                                            "title": _(u"Maintenance"),
                                            "link":  "portal_jalon_properties/gestion_maintenance"}]})

        if user_role == "Secretaire":
            menu_left.append({"id":       "gestion_pedagogique",
                              "class":    "",
                              "icon":     "fa fa-database",
                              "title":    _(u"Gestion pédagogique"),
                              "link":     "/portal_jalon_bdd/gestion_utilisateurs",
                              "sub_menu": []})

        return menu_left

    def getMonEspace(self):
        # LOG.info("----- getMonEspace -----")
        return {"site":        self.aq_parent.Title(),
                "grid":        self.getGridMonEspaceNew(),
                "maintenance": self.getPropertiesMaintenance(),
                "vidercache":  self.getJalonProperty("annoncer_vider_cache"),
                "messages":    self.getPropertiesMessages()}

    def generatePageMonEspace(self, request):
        # LOG.info("----- generatePageMonEspace -----")
        macro_mes_ressources_grid_generate = ["<metal:macro define-macro=\"mes_ressources_grid\">"]
        macro_mes_ressources_grid_generate.append(self.restrictedTraverse("portal_jalon_properties/macro_mes_ressources_grid_base")())
        macro_mes_ressources_grid_generate.append("</metal:macro>")
        self.restrictedTraverse("mes_ressources/macro_mes_ressources_grid").pt_edit("\n".join(macro_mes_ressources_grid_generate), "text/html", "utf-8")

        top_bar_base = self.restrictedTraverse("portal_jalon_properties/top_bar_base").PrincipiaSearchSource()

        request.set("left_menu", self.getTopBarMenuLeft('Manager'))
        top_bar_menu_left_base = self.restrictedTraverse("portal_jalon_properties/top_bar_menu_left_base")(REQUEST=request)
        top_bar_manager = top_bar_base.replace("*****left_menu*****", top_bar_menu_left_base)
        self.restrictedTraverse("top_bar/top_bar_manager").pt_edit(top_bar_manager, "text/html", "utf-8")

        request.set("left_menu", self.getTopBarMenuLeft('Personnel'))
        top_bar_menu_left_base = self.restrictedTraverse("portal_jalon_properties/top_bar_menu_left_base")(REQUEST=request)
        top_bar_personnel = top_bar_base.replace("*****left_menu*****", top_bar_menu_left_base)
        self.restrictedTraverse("top_bar/top_bar_personnel").pt_edit(top_bar_personnel, "text/html", "utf-8")

        request.set("left_menu", self.getTopBarMenuLeft('Secretaire'))
        top_bar_menu_left_base = self.restrictedTraverse("portal_jalon_properties/top_bar_menu_left_base")(REQUEST=request)
        top_bar_secretaire = top_bar_base.replace("*****left_menu*****", top_bar_menu_left_base)
        self.restrictedTraverse("top_bar/top_bar_secretaire").pt_edit(top_bar_secretaire, "text/html", "utf-8")

        request.set("left_menu", self.getTopBarMenuLeft('Etudiant'))
        top_bar_menu_left_base = self.restrictedTraverse("portal_jalon_properties/top_bar_menu_left_base")(REQUEST=request)
        top_bar_etudiant = top_bar_base.replace("*****left_menu*****", top_bar_menu_left_base)
        self.restrictedTraverse("top_bar/top_bar_etudiant").pt_edit(top_bar_etudiant, "text/html", "utf-8")
        self.restrictedTraverse("top_bar/top_bar_etudiantjalon").pt_edit(top_bar_etudiant, "text/html", "utf-8")

    #------------------------#
    # Fonctions de connexion #
    #------------------------#
    def getPropertiesConnexion(self, key=None):
        if key:
            return getattr(self, "_%s" % key)
        else:
            return {"activer_cas":            self._activer_cas,
                    "serviceUrl":             self._serviceUrl,
                    "casServerUrlPrefix":     self._casServerUrlPrefix,
                    "activer_creationcompte": self._activer_creationcompte}

    def setPropertiesConnexion(self, form):
        for key in form.keys():
            val = form[key]
            if key == "activer_cas":
                val = int(val)
            if key == "activer_creationcompte":
                val = int(val)
            setattr(self, "_%s" % key, val)

    #--------------------------------#
    # Fonctions du bloc Informations #
    #--------------------------------#
    def getPropertiesInfos(self, key=None):
        if key:
            return getattr(self, "_%s" % key)
        else:
            return {"activer_etablissement":   self._activer_etablissement,
                    "etablissement":           self._etablissement,
                    "lien_etablissement":      self._lien_etablissement,
                    "activer_lien_sesame":     self._activer_lien_sesame,
                    "lien_sesame":             self._lien_sesame,
                    "activer_lien_contact":    self._activer_lien_contact,
                    "lien_contact":            self._lien_contact,
                    "activer_lien_mention":    self._activer_lien_mention,
                    "lien_mention":            self._lien_mention,
                    "activer_lien_credit":     self._activer_lien_credit,
                    "lien_credit":             self._lien_credit, }

    def setPropertiesInfos(self, form):
        for key in form.keys():
            val = form[key]
            if key.startswith("activer_"):
                val = int(val)
            setattr(self, "_%s" % key, val)

    def getLienContact(self):
        """ Fournit un lien pour contacter l'administrateur du site."""
        portal = self.portal_url.getPortalObject()
        portal_title = portal.Title()
        if self._activer_lien_contact:
            lien_contact = self._lien_contact
        else:
            if self._activer_email_erreur:
                lien_contact = "mailto:%s" % self._adresse_email_erreur
            else:
                lien_contact = "mailto:%s" % portal.getProperty("email_from_address")
        return {"portal_title": portal_title,
                "contact_link":  lien_contact}

    #--------------------------------#
    # Fonctions du bloc Didacticiels #
    #--------------------------------#
    def getPropertiesDidacticiels(self, key=None):
        if key:
            return getattr(self, "_%s" % key)
        else:
            return {"activer_aide":            self._activer_aide,
                    "lien_aide":               self._lien_aide,
                    "activer_aide_plan":       self._activer_aide_plan,
                    "lien_aide_plan":          self._lien_aide_plan,
                    "activer_guide_anti_spam": self._activer_guide_anti_spam,
                    "message_guide_anti_spam": self._message_guide_anti_spam}

    def setPropertiesDidacticiels(self, form):
        for key in form.keys():
            val = form[key]
            if key.startswith("activer_"):
                val = int(val)
            setattr(self, "_%s" % key, val)

    #--------------------------------------------#
    # Fonctions du bloc de diffusion de messages #
    #--------------------------------------------#
    def getPropertiesMessages(self, key=None):
        if key:
            return getattr(self, "_%s" % key)
        else:
            return {"is_message_general": self._activer_message_general,
                    "message_general":    self._message_general,
                    "is_message_student": self._activer_bie,
                    "message_student":    self._bie_message,
                    "is_message_teacher": self._activer_message_enseignant,
                    "message_teacher":    self._message_enseignant}

    def setPropertiesMessages(self, form, request):
        # LOG.info("----- setPropertiesMessages -----")
        for key in form.keys():
            val = form[key]
            if key.startswith("activer_"):
                val = int(val)
            setattr(self, "_%s" % key, val)
        self.generateMessages(request)

    def generateMessages(self, request):
        # LOG.info("----- generateMessages -----")
        messages_properties = self.getPropertiesMessages()
        for key in messages_properties:
            request.set(key, messages_properties[key])

        #properties_messages = self.getPropertiesMaintenance()
        #for key in properties_messages:
        #    request.set(key, properties_messages[key])

        request.set("site", self.aq_parent.Title())
        request.set("maintenance", self.getPropertiesMaintenance())
        messages_template = ["<metal:macro define-macro=\"messages_folder_macro\">"]
        messages_template.append(self.restrictedTraverse("messages_folder/macro_messages_base")(REQUEST=request))
        messages_template.append("</metal:macro>metal:macro>")
        self.restrictedTraverse("messages_folder/Manager").pt_edit("\n".join(messages_template), "text/html", "utf-8")

        request.set("is_message_student", False)
        messages_template = ["<metal:macro define-macro=\"messages_folder_macro\">"]
        messages_template.append(self.restrictedTraverse("messages_folder/macro_messages_base")(REQUEST=request))
        messages_template.append("</metal:macro>metal:macro>")
        self.restrictedTraverse("messages_folder/Personnel").pt_edit("\n".join(messages_template), "text/html", "utf-8")

        request.set("is_message_teacher", False)
        request.set("is_message_student", messages_properties["is_message_student"])
        messages_template = ["<metal:macro define-macro=\"messages_folder_macro\">"]
        messages_template.append(self.restrictedTraverse("messages_folder/macro_messages_base")(REQUEST=request))
        messages_template.append("</metal:macro>metal:macro>")
        self.restrictedTraverse("messages_folder/Etudiant").pt_edit("\n".join(messages_template), "text/html", "utf-8")

    #-----------------------------#
    # Fonctions du bloc Courriels #
    #-----------------------------#
    def getPropertiesCourriels(self):
        return {"activer_erreur":          self._activer_erreur,
                "activer_email_erreur":    self._activer_email_erreur,
                "adresse_email_erreur":    self._adresse_email_erreur,
                "activer_liste_diffusion": self._activer_liste_diffusion,
                "type_diffusion":          self._type_diffusion,
                "format_diffusion":        self._format_diffusion}

    def setPropertiesCourriels(self, form):
        for key in form.keys():
            val = form[key]
            if key.startswith("activer_"):
                val = int(val)
            setattr(self, "_%s" % key, val)

    def isMailhost(self):
        mailhost = getToolByName(self, 'MailHost')
        if getattr(mailhost, 'smtp_host', ''):
            return True
        else:
            return False

    #-------------------------#
    # Fonctions de Mon Espace #
    #-------------------------#
    def getPropertiesMonEspace(self, key=None):
        if key:
            return getattr(self, "_%s" % key)
        else:
            return {"activer_fichiers":                  self._activer_fichiers,
                    "activer_presentations_sonorisees":  self._activer_presentations_sonorisees,
                    "activer_exercices_wims":            self._activer_exercices_wims,
                    "activer_liens":                     self._activer_liens,
                    "activer_liens_catalogue_bu":        self._activer_liens_catalogue_bu,
                    "activer_tags_catalogue_bu":         self._activer_tags_catalogue_bu,
                    "activer_lille1pod":                 self._activer_lille1pod,
                    "activer_termes_glossaire":          self._activer_termes_glossaire,
                    "activer_webconferences":            self._activer_webconferences,
                    "activer_lien_intracursus":          self._activer_lien_intracursus,
                    "lien_intracursus":                  self._lien_intracursus,
                    "activer_lien_assistance":           self._activer_lien_assistance,
                    "lien_assitance":                    self._lien_assitance,
                    "activer_video":                     self._activer_video,
                    "url_video":                         self._url_video,
                    "activer_vod":                       self._activer_vod}

    def getGridMonEspace(self, key=None):
        return [{"espace":      "fichiers",
                 "titre":       "Fichiers",
                 "activer":     self._activer_fichiers,
                 "repertoire":  "Fichiers",
                 "icone":       "fa fa-files-o"},
                {"espace":      "connect",
                 "titre":       "Présentations sonorisées",
                 "activer":     self._activer_presentations_sonorisees,
                 "repertoire":  "Sonorisation",
                 "icone":       "fa fa-microphone"},
                {"espace":      "wims",
                 "titre":       "Exercices Wims",
                 "activer":     self._activer_exercices_wims,
                 "repertoire":  "Wims",
                 "icone":       "fa fa-random"},
                {"espace":      "liens",
                 "titre":       "Ressources externes",
                 "activer":     self._activer_liens,
                 "repertoire":  "Externes",
                 "icone":       "fa fa-external-link"},
                {"espace":      "glossaire",
                 "titre":       "Termes de glossaire",
                 "activer":     self._activer_termes_glossaire,
                 "repertoire":  "Glossaire",
                 "icone":       "fa fa-font"},
                {"espace":      "connect",
                 "titre":       "Webconférences",
                 "activer":     self._activer_webconferences,
                 "repertoire":  "Webconference",
                 "icone":       "fa fa-headphones"},
                {"espace":      "video",
                 "titre":       "Vidéos",
                 "activer":     self._activer_lille1pod,
                 "repertoire":  "Video",
                 "icone":       "fa fa-youtube-play"},
                {"espace":      "vod",
                 "titre":       "VOD",
                 "activer":     self._activer_vod,
                 "repertoire":  "VOD",
                 "icone":       "fa fa-video-camera"}]

    def getGridMonEspaceNew(self, key=None):
        return [{"css":       "fichiers",
                 "title":     "Fichiers",
                 "activated": self._activer_fichiers,
                 "link":      "mes_ressources/mes_fichiers",
                 "icon":      "fa fa-files-o"},
                {"css":       "connect",
                 "title":     "Présentations sonorisées",
                 "activated": self._activer_presentations_sonorisees,
                 "link":      "mes_ressources/mes_presentations_sonorisees",
                 "icon":      "fa fa-microphone"},
                {"css":       "wims",
                 "title":     "Exercices Wims",
                 "activated": self._activer_exercices_wims,
                 "link":      "mes_ressources/mes_exercices_wims",
                 "icon":      "fa fa-random"},
                {"css":       "liens",
                 "title":     "Ressources externes",
                 "activated": self._activer_liens,
                 "link":      "mes_ressources/mes_ressources_externes",
                 "icon":      "fa fa-external-link"},
                {"css":       "glossaire",
                 "title":     "Termes de glossaire",
                 "activated": self._activer_termes_glossaire,
                 "link":      "mes_ressources/mes_termes_glossaire",
                 "icon":      "fa fa-font"},
                {"css":       "connect",
                 "title":     "Webconférences",
                 "activated": self._activer_webconferences,
                 "link":      "mes_ressources/mes_webconferences",
                 "icon":      "fa fa-headphones"},
                {"css":       "video",
                 "title":     "Vidéos",
                 "activated": self._activer_lille1pod,
                 "link":      "mes_ressources/mes_videos_pod",
                 "icon":      "fa fa-youtube-play"},
                {"css":       "vod",
                 "title":     "VOD",
                 "activated": self._activer_vod,
                 "link":      "mes_ressources/mes_vods",
                 "icon":      "fa fa-video-camera"}]

    def setPropertiesMonEspace(self, form, request):
        for key in form.keys():
            val = form[key]
            if key.startswith("activer_"):
                val = int(val)
            setattr(self, "_%s" % key, val)
        self.generatePageMonEspace(request)

    def isAdobeConnect(self):
        portal = self.portal_url.getPortalObject()
        connect = getattr(portal, "portal_connect", None)
        if connect:
            url_connexion = connect.getConnectProperty("url_connexion")
            if not url_connexion or url_connexion == "http://domainname.com/api/xml":
                return {"module":  False,
                        "message": "Le module Adobe Connect de Jalon n'est pas configuré"}
            else:
                return {"module":  True,
                        "message": "Test"}
        else:
            return {"module":  False,
                    "message": "Vous n'avez pas de module Adobe Connect installé dans Jalon"}

    def getPropertiesAdobeConnect(self):
        portal = self.portal_url.getPortalObject()
        connect = getattr(portal, "portal_connect", None)
        if connect:
            return {"url_connexion" : connect.getConnectProperty("url_connexion"),
                    "login"         : connect.getConnectProperty("login"),
                    "password"      : connect.getConnectProperty("password"),
                    "version"       : connect.getConnectProperty("version"),
                    "num_serveur"   : connect.getConnectProperty("num_serveur"),
                    "etablissement" : connect.getConnectProperty("etablissement"),
                    "dossiers"      : connect.getConnectProperty("dossiers"), }
        else:
            return None

    def setPropertiesAdobeConnect(self, form):
        portal = self.portal_url.getPortalObject()
        connect = getattr(portal, "portal_connect", None)
        connect.setProperties(form)

    def isWims(self):
        portal = self.portal_url.getPortalObject()
        wims = getattr(portal, "portal_wims", None)
        if wims:
            url_connexion = wims.getWimsProperty("url_connexion")
            if not url_connexion or url_connexion == "http://domainname.com/wims/wims.cgi":
                return {"module"  : False,
                        "message" : "Le module Wims de Jalon n'est pas configuré"}
            else:
                return {"module"  : True,
                        "message" : "Test"}
        else:
            return {"module"  : False,
                    "message" : "Vous n'avez pas de module Wims installé dans Jalon"}

    def getPropertiesWims(self):
        portal = self.portal_url.getPortalObject()
        wims = getattr(portal, "portal_wims", None)
        if wims:
            return {"url_connexion"       : wims.getWimsProperty("url_connexion"),
                    "login"               : wims.getWimsProperty("login"),
                    "password"            : wims.getWimsProperty("password"),
                    "classe_locale"       : wims.getWimsProperty("classe_locale"),
                    "nom_institution"     : wims.getWimsProperty("nom_institution"),
                    "donnees_classe"      : wims.getWimsProperty("donnees_classe"),
                    "donnees_superviseur" : wims.getWimsProperty("donnees_superviseur"),
                    "donnees_exercice"    : wims.getWimsProperty("donnees_exercice"), }
        else:
            return None

    def setPropertiesWims(self, form):
        portal = self.portal_url.getPortalObject()
        wims = getattr(portal, "portal_wims", None)
        wims.setProperties(form)

    def isPrimo(self):
        portal = self.portal_url.getPortalObject()
        primo = getattr(portal, "portal_primo", None)
        if primo:
            url_connexion = primo.getPrimoProperty("url_connexion")
            if not url_connexion or url_connexion == "http://domainname.com/PrimoWebServices/xservice/":
                return {"module"  : False,
                        "message" : "Le module ExLibris Primo de Jalon n'est pas configuré"}
            else:
                return {"module"  : True,
                        "message" : "Test"}
        else:
            return {"module"  : False,
                    "message" : "Vous n'avez pas de module ExLibris Primo installé dans Jalon"}

    def getPropertiesPrimo(self):
        portal = self.portal_url.getPortalObject()
        primo = getattr(portal, "portal_primo", None)
        if primo:
            return {"url_connexion"   : primo.getPrimoProperty("url_connexion"),
                    "url_catalogue"   : primo.getPrimoProperty("url_catalogue"),
                    "url_acquisition" : primo.getPrimoProperty("url_acquisition"), }
        else:
            return None

    def setPropertiesPrimo(self, form):
        portal = self.portal_url.getPortalObject()
        primo = getattr(portal, "portal_primo", None)
        primo.setProperties(form)

    #-------------------#
    # Fonctions Twitter #
    #-------------------#
    def getPropertiesTwitter(self, key=None):
        if key:
            return getattr(self, "_%s" % key)
        else:
            return {"COMPTE_TWITTER":     self._COMPTE_TWITTER,
                    "APP_KEY":            self._APP_KEY,
                    "APP_SECRET":         self._APP_SECRET,
                    "OAUTH_TOKEN":        self._OAUTH_TOKEN,
                    "OAUTH_TOKEN_SECRET": self._OAUTH_TOKEN_SECRET}

    def setPropertiesTwitter(self, form):
        for key in form.keys():
            setattr(self, "_%s" % key, form[key])

    def connexionTwitter(self):
        return Twython(self._APP_KEY, self._APP_SECRET, self._OAUTH_TOKEN, self._OAUTH_TOKEN_SECRET)

    def getTwitterCours(self, hashtag):
        twitter = self.connexionTwitter()
        try:
            tweets = twitter.search(q='from:%s+%s' % (self._COMPTE_TWITTER, hashtag), count=50)
            liste = []
            for tweet in tweets['statuses']:
                #print 'Tweet from @%s Date: %s' % (tweet['user']['screen_name'].encode('utf-8'), tweet['created_at'])
                #print tweet['text'].encode('utf-8')
                liste.append({"date": DateTime(tweet['created_at'], datefmt='international').strftime("%d/%m/%Y %H:%M"),
                              "texte": tweet['text']})
            return liste
        except TwythonError as e:
            print e

    #----------------------------------------------------#
    # Fonctions de récupération des données utilisateurs #
    #----------------------------------------------------#
    def getPropertiesDonneesUtilisateurs(self, key=None):
        if key:
            return getattr(self, "_%s" % key)
        else:
            return {"activer_ldap":           self._activer_ldap,
                    "base_ldap":              self._base_ldap,
                    "schema_ldap":            self._schema_ldap,
                    "fiche_ldap":             self._fiche_ldap,
                    "activer_trombinoscope":  self._activer_trombinoscope,
                    "activer_stockage_photo": self._activer_stockage_photo,
                    "lien_trombinoscope":     self._lien_trombinoscope}

    def setPropertiesDonneesUtilisateurs(self, form):
        for key in form.keys():
            val = form[key]
            if key.startswith("activer_"):
                val = int(val)
            setattr(self, "_%s" % key, val)

    def isLDAP(self):
        return jalon_utils.isLDAP()

    def rechercherUtilisateur(self, username, typeUser, match=False, attribut="displayName"):
        """recherche un Utilisateur."""
        if self.isLDAP():
            if self._schema_ldap == "supann":
                ldap = "ldap-plugin"
                if typeUser == "Etudiant":
                    ldap = "ldap-plugin-etu"
                return self.rechercherUserLDAPSupann(username, attribut, ldap, match)
            if self._schema_ldap == "eduPerson":
                return self.rechercherUserLDAPEduPerson(username, attribut, "ldap-plugin", match)
        else:
            portal_jalon_bdd = getToolByName(self, "portal_jalon_bdd")
            return portal_jalon_bdd.rechercherUtilisateursByName(username, typeUser)

    def rechercherUserLDAPSupann(self, username, attribut, ldap="ldap-plugin", match=False):
        retour = []
        acl_users = getattr(getattr(getattr(self, "acl_users"), ldap), "acl_users")
        for user in acl_users.findUser(search_param=attribut, search_term=username, exact_match=match):
            if "supannAliasLogin" in user:
                email = user["supannAliasLogin"]
                if "mail" in user:
                    email = user["mail"].decode("iso-8859-1")
                retour.append({"id":    user["supannAliasLogin"],
                               "name":  user["displayName"].decode("iso-8859-1"),
                               "email": email})
        return json.dumps(retour)

    def rechercherUserLDAPEduPerson(self, username, attribut, ldap="ldap-plugin", match=False):
        retour = []
        eduPersonAffiliation = ["employee", "faculty"]
        if ldap == "ldap-plugin-etu":
            eduPersonAffiliation = ["student"]
        acl_users = getattr(getattr(getattr(self, "acl_users"), ldap), "acl_users")
        for user in acl_users.findUser(search_param=attribut, search_term=username, exact_match=match):
            if "login" in user and user["eduPersonAffiliation"] in eduPersonAffiliation:
                email = user["login"]
                if "mail" in user:
                    email = user["mail"].decode("iso-8859-1")
                retour.append({"id":    user["login"],
                               "name":  user["displayName"].decode("iso-8859-1"),
                               "email": email})
        return json.dumps(retour)

    #-------------------------------#
    # Fonctions de Google Analytics #
    #-------------------------------#
    def getPropertiesGA(self, key=None):
        if key:
            return getattr(self, "_%s" % key)
        else:
            return {"activer_ga":    self._activer_ga,
                    "ga_id_account": self._ga_id_account,
                    "ga_id_domain":  self._ga_id_domain,
                    "ga_cryptage":   self._ga_cryptage}

    def setPropertiesGA(self, form):
        for key in form.keys():
            val = form[key]
            if key == "activer_ga":
                val = int(val)
            setattr(self, "_%s" % key, val)

    #--------------------------#
    # Fonctions de Maintenance #
    #--------------------------#
    def getPropertiesMaintenance(self):
        date_debut_maintenance = self._date_debut_maintenance
        if not date_debut_maintenance:
            date_debut_maintenance = DateTime()
        date_fin_maintenance = self._date_fin_maintenance
        if not date_fin_maintenance:
            date_fin_maintenance = DateTime()
        return {"annoncer_maintenance":       self._annoncer_maintenance,
                "activer_maintenance":        self._activer_maintenance,
                "date_debut_maintenance":     date_debut_maintenance,
                "date_debut_maintenance_aff": jalon_utils.getLocaleDate(date_debut_maintenance, '%d %B %Y - %Hh%M'),
                "date_fin_maintenance":       date_fin_maintenance,
                "date_fin_maintenance_aff":   jalon_utils.getLocaleDate(date_fin_maintenance, '%d %B %Y - %Hh%M'),
                "annoncer_vider_cache":       self._annoncer_vider_cache,
                "url_news_maintenance":       self._url_news_maintenance}

    def setPropertiesMaintenance(self, form, request):
        # LOG.info("----- setPropertiesMaintenance -----")
        for key in form.keys():
            val = form[key]
            if key.startswith("activer_") or key.startswith("annoncer_"):
                val = int(val)
            if key.startswith("date_"):
                val = DateTime(val)
            setattr(self, "_%s" % key, val)
        self.generatePageMonEspace(request)
        self.generateMessages(request)

    #------------------------#
    # Fonction Elasticsearch #
    #------------------------#
    def getPropertiesElasticsearch(self):
        portal = self.portal_url.getPortalObject()
        portal_elasticsearch = getattr(portal, "portal_jalon_elasticsearch", None)
        return portal_elasticsearch.getPropertiesElasticsearch()

    def setPropertiesElasticsearch(self, propertiesElasticsearch):
        portal = self.portal_url.getPortalObject()
        portal_elasticsearch = getattr(portal, "portal_jalon_elasticsearch", None)
        portal_elasticsearch.setPropertiesElasticsearch(propertiesElasticsearch)

    def searchElasticsearch(self, type_search=None, term_search=None, page=1):
        portal = self.portal_url.getPortalObject()
        portal_elasticsearch = getattr(portal, "portal_jalon_elasticsearch", None)
        return portal_elasticsearch.searchElasticsearch(type_search, term_search, page)

    def isElasticsearch(self):
        portal = self.portal_url.getPortalObject()
        portal_jalon_elasticsearch = getattr(portal, "portal_jalon_elasticsearch", None)
        if portal_jalon_elasticsearch:
            elasticsearch_properties = portal_jalon_elasticsearch.getPropertiesElasticsearch()
            if not elasticsearch_properties["url_connexion"] or elasticsearch_properties["url_connexion"] == "http://domainname.com/":
                return {"module":  False,
                        "message": "Le module Elasticsearch de Jalon n'est pas configuré"}
            else:
                return {"module":  True,
                        "message": "Test"}
        else:
            return {"module":  False,
                    "message": "Vous n'avez pas de module Elasticsearch installé dans Jalon"}

    #----------------#
    # Fonction Wowza #
    #----------------#
    def isVOD(self):
        portal = self.portal_url.getPortalObject()
        portal_jalon_wowza = getattr(portal, "portal_jalon_wowza", None)
        if portal_jalon_wowza:
            wowza_properties = portal_jalon_wowza.getWowzaProperties()
            if not wowza_properties["wowza_server"] or wowza_properties["wowza_server"] == "http://domainname.com":
                return {"module":  False,
                        "message": "Le module Wowza de Jalon n'est pas configuré"}
            else:
                return {"module":  True,
                        "message": "Test"}
        else:
            return {"module":  False,
                    "message": "Vous n'avez pas de module Wowza installé dans Jalon"}

    def getWowzaProperties(self):
        portal = self.portal_url.getPortalObject()
        portal_jalon_wowza = getattr(portal, "portal_jalon_wowza", None)
        return portal_jalon_wowza.getWowzaProperties()

    def setWowzaProperties(self, wowzaProperties):
        portal = self.portal_url.getPortalObject()
        portal_jalon_wowza = getattr(portal, "portal_jalon_wowza", None)
        portal_jalon_wowza.setWowzaProperties(wowzaProperties)

    def searchExtraits(self, page, term_search):
        portal = self.portal_url.getPortalObject()
        portal_jalon_wowza = getattr(portal, "portal_jalon_wowza", None)
        return portal_jalon_wowza.searchExtraits(page, term_search)

    def getExpirationDate(self, pod):
        portal = self.portal_url.getPortalObject()
        portal_jalon_wowza = getattr(portal, "portal_jalon_wowza", None)
        return portal_jalon_wowza.getStreamingAvailable(pod)

    def modifyStreaming(self, params):
        # LOG.info("----- modifyStreaming -----")
        # LOG.info(params)
        portal = self.portal_url.getPortalObject()
        portal_jalon_wowza = getattr(portal, "portal_jalon_wowza", None)
        if "datetime-expiration_date" in params:
            portal_jalon_wowza.modifyStreaming(params["pod"], params["datetime-expiration_date"])
        else:
            portal_jalon_wowza.modifyStreaming(params["pod"])

    # -----------------------#
    # Gestion des catégories #
    #------------------------#
    def getCategorie(self, key=None):
        if key:
            return self._categories.get(int(key), None)
        return dict(self._categories)

    def getClefCategories(self):
        clefs = self._categories.keys()[:]
        clefs.sort(lambda x, y: cmp(int(x), int(y)))
        return clefs

    def setCategories(self, categories):
        if type(self._categories).__name__ != "PersistentMapping":
            self._categories = PersistentDict(categories)
        else:
            self._categories = categories

    def getUsersCategorie(self, clef):
        categorie = self.getCategorie(int(clef))
        if categorie["users"] == ["all"]:
            return ["all"]
        return self.getInfosUsers(categorie["users"])

    def setUsersCategorie(self, form):
        categories = copy.deepcopy(self.getCategorie())
        if "all" in form:
            categories[int(form["clef"])]["users"] = ["all"]
        else:
            users = []
            if "users-actu" in form:
                users = form["users-actu"]
            if "username" in form:
                usernames = form["username"].split(",")
                if usernames != ['']:
                    for username in usernames:
                        if not username in users:
                            users.append(username)
            categories[int(form["clef"])]["users"] = users
        self.setCategories(categories)

    def getInfosUsers(self, users):
        infosUsers = []
        for user in users:
            result = self.rechercherUtilisateur(user, "Personnel", match=True, attribut="supannAliasLogin")
            infosUsers.append(json.loads(result)[0])
        infosUsers.sort(lambda x, y: cmp(x["name"], y["name"]))
        return infosUsers

    def creerCategorie(self, title):
        clefs = copy.deepcopy(self.getClefCategories())
        clef = int(clefs[-1]) + 1
        categories = self.getCategorie()
        categories[clef] = {"title": title, "users": []}
        self.setCategories(categories)

    def renommerCategorie(self, clef, title):
        categories = copy.deepcopy(self.getCategorie())
        categories[int(clef)]["title"] = title
        self.setCategories(categories)

    #--------------------#
    # Gestion du Podcast #
    #--------------------#
    def getVariablesPodcast(self):
        return {"activerPodcasts": self._activerPodcasts,
                "uploadPodcasts":  self._uploadPodcasts,
                "dnsPodcasts":     self._dnsPodcasts}

    def setVariablesPodcast(self, podcasts):
        if "activerPodcasts" in podcasts:
            self._activerPodcasts = podcasts["activerPodcasts"]
        self._uploadPodcasts = podcasts["uploadPodcasts"]
        self._dnsPodcasts = podcasts["dnsPodcasts"]

    def getUsersPodcast(self, tiny=None):
        if tiny:
            return self._dicoUsersPodcast.keys()
        listeUsers = []
        for user in self._dicoUsersPodcast.keys():
            listeUsers.append({"id": user,
                               "fullname": jalon_utils.getIndividu(user, "dict")["fullname"]})
        listeUsers.sort(lambda x, y: cmp(x["fullname"], y["fullname"]))
        return listeUsers

    def setDicoUsersPodcast(self, dicoUsersPodcast):
        if type(self._dicoUsersPodcast).__name__ != "PersistentMapping":
            self._dicoUsersPodcast = PersistentDict(dicoUsersPodcast)
        else:
            self._dicoUsersPodcast = dicoUsersPodcast

    def setUsersPodcast(self, form):
        dicoUsersPodcast = self._dicoUsersPodcast
        if "username" in form:
            usernames = form["username"].split(",")
            if usernames != ['']:
                for username in usernames:
                    if not username in dicoUsersPodcast.keys():
                        dicoUsersPodcast[username] = {}
                self.setDicoUsersPodcast(dicoUsersPodcast)

    def setUsersiTunesU(self, auteurCours):
        dicoUsersPodcast = self._dicoUsersPodcast
        if not auteurCours in dicoUsersPodcast:
            dicoUsersPodcast[auteurCours] = []
            self.setDicoUsersPodcast(dicoUsersPodcast)

    def setCoursUser(self, auteurCours, idCours):
        dicoUsersPodcast = self._dicoUsersPodcast
        listeCours = dicoUsersPodcast[auteurCours]
        if not idCours in listeCours:
            listeCours.append(idCours)
            dicoUsersPodcast[auteurCours] = listeCours
            self.setDicoUsersPodcast(dicoUsersPodcast)

    def getCoursUser(self, user):
        retour = []
        portal = self.aq_parent
        listeCours = self._dicoUsersPodcast[user]
        dicoLibCatITunesU = self._dicoLibCatITunesU
        repCours = getattr(getattr(portal, "cours"), user)
        for idCours in listeCours:
            cours = getattr(repCours, idCours, None)
            if cours:
                catiTunesU = cours.getCatiTunesU()
                if catiTunesU:
                    catPrinCours = dicoLibCatITunesU[catiTunesU[:3]]
                    catSecCours = dicoLibCatITunesU[catiTunesU]
                else:
                    catPrinCours = "Erreur : catégorie à redéfinir"
                    catSecCours = ""
                retour.append({"idCours":      idCours,
                               "titreCours":   cours.Title(),
                               "etatCours":    cours.isDiffuseriTunesU(),
                               "catPrinCours": catPrinCours,
                               "catSecCours":  catSecCours,
                               "urlCours":     cours.absolute_url()})

        retour.sort(lambda x, y: cmp(x["titreCours"], y["titreCours"]))
        return retour

    def getAffCatiTunesUCours(self, idCatiTunesU):
        dicoLibCatITunesU = self._dicoLibCatITunesU
        catPrinCours = dicoLibCatITunesU[idCatiTunesU[:3]]
        catSecCours = dicoLibCatITunesU[idCatiTunesU]
        return "%s / %s" % (catPrinCours, catSecCours)

    def validerCoursiTunesU(self, idCours, auteurCours):
        portal = self.aq_parent
        repCours = getattr(getattr(portal, "cours"), auteurCours)
        cours = getattr(repCours, idCours, None)
        if idCours:
            cours.setProperties({"diffusioniTunesU": True})
            cours.setProperties({"DateDerniereModif": DateTime()})
            infosMembre = jalon_utils.getIndividu(auteurCours, "dict")
            jalon_utils.envoyerMail({"a":       infosMembre["email"],
                                     "objet":   "Validation de votre demande iTunesU",
                                     "message": "Bonjour\n\nLe cours \"%s\" pour lequel vous avez demandé une publication sur iTunesU a été validé dans la catégorie : \"%s\".\n\nCordialement,\nL'équipe %s" % (cours.Title(), cours.getAffCatiTunesUCours(), portal.Title()), })

    def rejeterCoursiTunesU(self, idCours, auteurCours):
        portal = self.aq_parent
        repCours = getattr(getattr(portal, "cours"), auteurCours)
        cours = getattr(repCours, idCours, None)
        if idCours:
            infosMembre = jalon_utils.getIndividu(auteurCours, "dict")
            jalon_utils.envoyerMail({"a":        infosMembre["email"],
                                     "objet":    "Rejet de votre demande iTunesU",
                                     "message": "Bonjour\n\nLe cours \"%s\" pour lequel vous avez demandé une publication sur iTunesU dans la catégorie : \"%s\" a été rejeté.\n\nNous vous remercions de votre compréhension,\n\nCordialement,\nL'équipe %s" % (cours.Title(), cours.getAffCatiTunesUCours(), portal.Title()), })
            cours.setProperties({"DiffusioniTunesU": False})
            cours.setProperties({"CatiTunesU": ""})
            cours.setProperties({"DateDerniereModif": DateTime()})
            dicoUsersPodcast = self._dicoUsersPodcast
            listeCours = dicoUsersPodcast[auteurCours]
            listeCours.remove(idCours)
            dicoUsersPodcast[auteurCours] = listeCours
            self.setDicoUsersPodcast(dicoUsersPodcast)

    def getCatITunesU(self):
        return self._catITunesU

    def getPropertiesCatiTunesU(self):
        return {"dicoLibCatITunesU" : self._dicoLibCatITunesU,
                "catITunesU"        : self._catITunesU,
                "ordreCatiTunesU"   : self._ordreCatiTunesU}

    def getListeCatITunesU(self):
        listeCatITunesU = self._catITunesU.keys()
        listeCatITunesU.sort()
        return listeCatITunesU

    #-------------------------------#
    # Fonction Communication NodeJS #
    #-------------------------------#
    def getPropertiesCommunication(self):
        portal = self.portal_url.getPortalObject()
        portal_jalon_communication = getattr(portal, "portal_jalon_communication", None)
        return portal_jalon_communication.getPropertiesCommunication()

    def setPropertiesCommunication(self, propertiesCommunication):
        portal = self.portal_url.getPortalObject()
        portal_jalon_communication = getattr(portal, "portal_jalon_communication", None)
        portal_jalon_communication.setPropertiesCommunication(propertiesCommunication)

    #----------------------#
    # Fonction Breadcrumbs #
    #----------------------#
    def getBreadcrumbs(self, page="Basic"):
        return self.__getattribute__("getBreadcrumbs%s" % page)()

    def getBreadcrumbsBasic(self):
        return [{"title": _(u"Configuration de Jalon"),
                 "icon":  "fa fa-cogs",
                 "link":  "%s/portal_jalon_properties/@@jalon_properties" % self.absolute_url()}]

    def getBreadcrumbsConnexion(self):
        return [{"title": _(u"Configuration de Jalon"),
                 "icon":  "fa fa-cogs",
                 "link":  "%s/portal_jalon_properties/@@jalon_properties" % self.absolute_url()},
                {"title": _(u"Connexion à Jalon"),
                 "icon":  "fa fa-key",
                 "link":  "%s/portal_jalon_properties/gestion_connexion" % self.absolute_url()}]

    def getBreadcrumbsMonEspace(self):
        return [{"title": _(u"Configuration de Jalon"),
                 "icon":  "fa fa-cogs",
                 "link":  "%s/portal_jalon_properties/@@jalon_properties" % self.absolute_url()},
                {"title": _(u"Gestion \"Mes ressources\""),
                 "icon":  "fa fa-folder-open",
                 "link":  "%s/portal_jalon_properties/gestion_mes_ressources" % self.absolute_url()}]

    def getBreadcrumbsMesCours(self):
        return [{"title": _(u"Configuration de Jalon"),
                 "icon":  "fa fa-cogs",
                 "link":  "%s/portal_jalon_properties/@@jalon_properties" % self.absolute_url()},
                {"title": _(u"Gestion des Cours"),
                 "icon":  "fa fa-university",
                 "link":  "%s/portal_jalon_properties/gestion_mes_cours" % self.absolute_url()}]

    def getBreadcrumbsInformations(self):
        return [{"title": _(u"Configuration de Jalon"),
                 "icon":  "fa fa-cogs",
                 "link":  "%s/portal_jalon_properties/@@jalon_properties" % self.absolute_url()},
                {"title": _(u"Gestion des liens d'informations"),
                 "icon":  "fa fa-external-link-square",
                 "link":  "%s/portal_jalon_properties/gestion_infos" % self.absolute_url()}]

    def getBreadcrumbsDidacticiels(self):
        return [{"title": _(u"Configuration de Jalon"),
                 "icon":  "fa fa-cogs",
                 "link":  "%s/portal_jalon_properties/@@jalon_properties" % self.absolute_url()},
                {"title": _(u"Gestion des didacticiels"),
                 "icon":  "fa fa-life-ring",
                 "link":  "%s/portal_jalon_properties/gestion_didacticiels" % self.absolute_url()}]

    def getBreadcrumbsMessages(self):
        return [{"title": _(u"Configuration de Jalon"),
                 "icon":  "fa fa-cogs",
                 "link":  "%s/portal_jalon_properties/@@jalon_properties" % self.absolute_url()},
                {"title": _(u"Gestion de la diffusion de messages"),
                 "icon":  "fa fa-bullhorn",
                 "link":  "%s/portal_jalon_properties/gestion_messages" % self.absolute_url()}]

    def getBreadcrumbsEmail(self):
        return [{"title": _(u"Configuration de Jalon"),
                 "icon":  "fa fa-cogs",
                 "link":  "%s/portal_jalon_properties/@@jalon_properties" % self.absolute_url()},
                {"title": _(u"Gestion des paramètres de courriels"),
                 "icon":  "fa fa-envelope-o",
                 "link":  "%s/portal_jalon_properties/gestion_email" % self.absolute_url()}]

    def getBreadcrumbsUsers(self):
        return [{"title": _(u"Configuration de Jalon"),
                 "icon":  "fa fa-cogs",
                 "link":  "%s/portal_jalon_properties/@@jalon_properties" % self.absolute_url()},
                {"title": _(u"Gestion des données utilisateurs"),
                 "icon":  "fa fa-users",
                 "link":  "%s/portal_jalon_properties/gestion_donnees_utilisateurs" % self.absolute_url()}]

    def getBreadcrumbsGA(self):
        return [{"title": _(u"Configuration de Jalon"),
                 "icon":  "fa fa-cogs",
                 "link":  "%s/portal_jalon_properties/@@jalon_properties" % self.absolute_url()},
                {"title": _(u"Gestion de Google Analytics"),
                 "icon":  "fa fa-line-chart",
                 "link":  "%s/portal_jalon_properties/gestion_ga" % self.absolute_url()}]

    def getBreadcrumbsMaintenance(self):
        return [{"title": _(u"Configuration de Jalon"),
                 "icon":  "fa fa-cogs",
                 "link":  "%s/portal_jalon_properties/@@jalon_properties" % self.absolute_url()},
                {"title": _(u"Gestion des paramètres de maintenance"),
                 "icon":  "fa fa-umbrella",
                 "link":  "%s/portal_jalon_properties/gestion_maintenance" % self.absolute_url()}]



## Controller Python Script "saveConfigTwitter"
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=
##title=Cours edit
##

form = context.REQUEST.form
del form["form.button.save"]
del form["form.submitted"]
context.setPropertiesTwitter(form)
context.REQUEST.RESPONSE.redirect("%s/@@jalon-configuration?gestion=gestion_twitter" % context.absolute_url())
